@@ -19,7 +19,7 @@
  *
  */
 
-/** Kamailio core :: main file (init, daemonize, startup) 
+/** Kamailio core :: main file (init, daemonize, startup)
  * @file main.c
  * @ingroup core
  * Module: core
@@ -133,6 +133,7 @@
 #include "async_task.h"
 #include "dset.h"
 #include "timer_proc.h"
+#include "srapi.h"
 
 #ifdef DEBUG_DMALLOC
 #include <dmalloc.h>
@@ -509,10 +510,10 @@ char *sr_memmng_pkg = NULL;
 char *sr_memmng_shm = NULL;
 
 /* call it before exiting; if show_status==1, mem status is displayed */
-void cleanup(show_status)
+void cleanup(int show_status)
 {
 	int memlog;
-	
+
 	/*clean-up*/
 #ifndef SHM_SAFE_MALLOC
 	if (_shm_lock)
@@ -708,7 +709,7 @@ void handle_sigs(void)
 				LOG(memlog, "Memory status (pkg):\n");
 				pkg_status();
 			}
-			if (cfg_get(core, core_cfg, mem_summary) & 2) {
+			if (cfg_get(core, core_cfg, mem_summary) & 4) {
 				LOG(memlog, "Memory still-in-use summary (pkg):\n");
 				pkg_sums();
 			}
@@ -716,11 +717,11 @@ void handle_sigs(void)
 #endif
 #ifdef SHM_MEM
 		if (memlog <= cfg_get(core, core_cfg, debug)){
-			if (cfg_get(core, core_cfg, mem_summary) & 1) {
+			if (cfg_get(core, core_cfg, mem_summary) & 2) {
 				LOG(memlog, "Memory status (shm):\n");
 				shm_status();
 			}
-			if (cfg_get(core, core_cfg, mem_summary) & 2) {
+			if (cfg_get(core, core_cfg, mem_summary) & 8) {
 				LOG(memlog, "Memory still-in-use summary (shm):\n");
 				shm_sums();
 			}
@@ -820,7 +821,7 @@ void sig_usr(int signo)
 							LOG(memlog, "Memory status (pkg):\n");
 							pkg_status();
 						}
-						if (cfg_get(core, core_cfg, mem_summary) & 2) {
+						if (cfg_get(core, core_cfg, mem_summary) & 4) {
 							LOG(memlog, "Memory still-in-use summary (pkg):"
 									"\n");
 							pkg_sums();
@@ -831,7 +832,20 @@ void sig_usr(int signo)
 					_exit(0);
 					break;
 			case SIGUSR1:
-				/* statistics, do nothing, printed only from the main proc */
+#ifdef PKG_MALLOC
+					cfg_update_no_cbs();
+					memlog=cfg_get(core, core_cfg, memlog);
+					if (memlog <= cfg_get(core, core_cfg, debug)){
+						if (cfg_get(core, core_cfg, mem_summary) & 1) {
+							LOG(memlog, "Memory status (pkg):\n");
+							pkg_status();
+						}
+						if (cfg_get(core, core_cfg, mem_summary) & 4) {
+							LOG(memlog, "Memory still-in-use summary (pkg):\n");
+							pkg_sums();
+						}
+					}
+#endif
 					break;
 				/* ignored*/
 			case SIGUSR2:
@@ -1836,6 +1850,7 @@ int main(int argc, char** argv)
 	debug_flag=0;
 	dont_fork_cnt=0;
 
+	sr_cfgenv_init();
 	daemon_status_init();
 
 	dprint_init_colors();
