@@ -1,21 +1,14 @@
 /*
- * $Id$
- *
  * Copyright (C) 2006 iptelorg GmbH
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -24,10 +17,6 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/* History:
- * --------
- *  2006-02-08  created by andrei
- */
 
 
 #include "binrpc.h"
@@ -47,8 +36,9 @@
    rpc->scan (default: not set) */
 int autoconvert=0;
 
-int binrpc_max_body_size = 4; /* multiplied by 1024 in mod init */
-int  binrpc_struct_max_body_size = 1; /* multiplied by 1024 in mod init */
+int binrpc_max_body_size = 32; /* multiplied by 1024 in mod init */
+int binrpc_struct_max_body_size = 8; /* multiplied by 1024 in mod init */
+
 #define BINRPC_MAX_BODY	binrpc_max_body_size  /* maximum body for send */
 #define STRUCT_MAX_BODY	binrpc_struct_max_body_size
 #define MAX_MSG_CHUNKS	96
@@ -496,7 +486,7 @@ static void rpc_fault(struct binrpc_ctx* ctx, int code, char* fmt, ...)
 	va_end(ap);
 
 	len++; /* vnsprintf doesn't include the terminating 0 */
-	return _rpc_fault(ctx, code, buf, len);
+	_rpc_fault(ctx, code, buf, len);
 }
 
 /* Prepare the error reply without sending out the message */
@@ -513,7 +503,7 @@ static int rpc_fault_prepare(struct binrpc_ctx* ctx, int code, char* fmt, ...)
 	}
 	va_start(ap, fmt);
 	len=vsnprintf(buf, MAX_FAULT_LEN, fmt, ap); /* ignore trunc. errors */
-	if ((len<0) || (len > MAX_FAULT_LEN))
+	if ((len<0) || (len >= MAX_FAULT_LEN))
 		len=MAX_FAULT_LEN-1;
 	va_end(ap);
 
@@ -666,7 +656,7 @@ int process_rpc_req(unsigned char* buf, int size, int* bytes_needed,
 	
 	/* get rpc method */
 	val.type=BINRPC_T_STR;
-	f_ctx.in.s=binrpc_read_record(ctx, f_ctx.in.s, f_ctx.in.end, &val, &err);
+	f_ctx.in.s=binrpc_read_record(ctx, f_ctx.in.s, f_ctx.in.end, &val, 0, &err);
 	if (err<0){
 		LOG(L_CRIT, "ERROR: bad rpc request method, binrpc error: %s (%d)\n",
 				binrpc_error(err), err);
@@ -866,9 +856,10 @@ static int rpc_scan(struct binrpc_ctx* ctx, char* fmt, ...)
 			case 'b': /* bool */
 			case 't': /* time */
 			case 'd': /* int */
+			case 'u': /* uint */
 				v.type=autoconv?BINRPC_T_ALL:BINRPC_T_INT;
 				ctx->in.s=binrpc_read_record(&ctx->in.ctx, ctx->in.s,
-												ctx->in.end, &v, &err);
+												ctx->in.end, &v, 0, &err);
 				if (err<0 || ((i=binrpc_val_conv_int(&v, &err))==0 && err<0))
 						goto error_read;
 				*(va_arg(ap, int*))=i;
@@ -876,7 +867,7 @@ static int rpc_scan(struct binrpc_ctx* ctx, char* fmt, ...)
 			case 'f':
 				v.type=autoconv?BINRPC_T_ALL:BINRPC_T_DOUBLE;
 				ctx->in.s=binrpc_read_record(&ctx->in.ctx, ctx->in.s,
-												ctx->in.end, &v, &err);
+												ctx->in.end, &v, 0, &err);
 				if (err<0 || ((d=binrpc_val_conv_double(&v, &err))==0 &&
 						err<0))
 					goto error_read;
@@ -886,7 +877,7 @@ static int rpc_scan(struct binrpc_ctx* ctx, char* fmt, ...)
 			case 'S': /* str */
 				v.type=autoconv?BINRPC_T_ALL:BINRPC_T_STR;
 				ctx->in.s=binrpc_read_record(&ctx->in.ctx, ctx->in.s, 
-												ctx->in.end, &v,&err);
+												ctx->in.end, &v, 0, &err);
 				if (err<0 || ((s=binrpc_val_conv_str(ctx, &v, &err))==0 &&
 							err<0)){
 					v.u.strval.s="if you get this string, you don't"
@@ -906,7 +897,7 @@ static int rpc_scan(struct binrpc_ctx* ctx, char* fmt, ...)
 				/* FIXME: structure reading doesn't work for now */
 #if 0
 				ctx->in.s=binrpc_read_record(&ctx->in.ctx, ctx->in.s, 
-												ctx->in.end, &v, &err);
+												ctx->in.end, &v, 0, &err);
 				if (err<0) goto error_read;
 				ctx->in.in_struct++;
 				*(va_arg(ap, void**))=ctx; /* use the same context */
@@ -965,6 +956,7 @@ static int rpc_add(struct binrpc_ctx* ctx, char* fmt, ...)
 			case 'd':
 			case 't':
 			case 'b':
+			case 'u':
 				err=binrpc_addint(&ctx->out.pkt, va_arg(ap, int));
 				if (err<0) goto error_add;
 				break;
@@ -1071,6 +1063,7 @@ static int rpc_struct_add(struct rpc_struct_l* s, char* fmt, ...)
 			case 'd':
 			case 't':
 			case 'b':
+			case 'u':
 				avp.type=BINRPC_T_INT;
 				avp.u.intval=va_arg(ap, int);
 				break;
@@ -1134,6 +1127,7 @@ static int rpc_array_add(struct rpc_struct_l* s, char* fmt, ...)
 			case 'd':
 			case 't':
 			case 'b':
+			case 'u':
 				err=binrpc_addint(&s->pkt, va_arg(ap, int));
 				if (err<0) goto error_add;
 				break;
