@@ -449,7 +449,7 @@ int db_mongodb_get_columns(const db1_con_t* _h, db1_res_t* _r)
 #endif
 
 			default:
-				LM_WARN("unhandled data type column (%.*s) type id (%d), "
+				LM_INFO("unhandled data type column (%.*s) type id (%d), "
 						"use DB1_STRING as default\n", RES_NAMES(_r)[col]->len,
 						RES_NAMES(_r)[col]->s, coltype);
 				RES_TYPES(_r)[col] = DB1_STRING;
@@ -470,7 +470,7 @@ int db_mongodb_get_columns(const db1_con_t* _h, db1_res_t* _r)
  * \param _r database result set
  * \return 0 on success, negative on failure
  */
-static int db_mongdob_convert_bson(const db1_con_t* _h, db1_res_t* _r,
+static int db_mongodb_convert_bson(const db1_con_t* _h, db1_res_t* _r,
 		int _row, const bson_t *_rdoc)
 {
 	static str dummy_string = {"", 0};
@@ -522,9 +522,15 @@ static int db_mongdob_convert_bson(const db1_con_t* _h, db1_res_t* _r,
 		LM_DBG("looking for field[%d] named: %s\n", col, colname);
 		if(mgres->colsdoc) {
 			if(!bson_iter_find(&riter, colname)) {
-				LM_ERR("field [%s] not found in result iterator\n",
+				if (!bson_iter_init (&riter, _rdoc)) {
+					LM_ERR("failed to initialize result iterator\n");
+					return -3;
+				}
+				if(!bson_iter_find(&riter, colname)) {
+					LM_ERR("field [%s] not found in result iterator\n",
 						colname);
-				return -4;
+					return -4;
+				}
 			}
 			piter = &riter;
 		} else {
@@ -619,7 +625,7 @@ static int db_mongdob_convert_bson(const db1_con_t* _h, db1_res_t* _r,
  * \param _r database result set
  * \return 0 on success, negative on failure
  */
-static int db_mongdob_convert_result(const db1_con_t* _h, db1_res_t* _r)
+static int db_mongodb_convert_result(const db1_con_t* _h, db1_res_t* _r)
 {
 	int row;
 	db_mongodb_result_t *mgres;
@@ -671,7 +677,7 @@ static int db_mongdob_convert_result(const db1_con_t* _h, db1_res_t* _r)
 			LM_DBG("selected document: %s\n", jstr);
 			bson_free (jstr);
 		}
-		if(db_mongdob_convert_bson(_h, _r, row, itdoc)) {
+		if(db_mongodb_convert_bson(_h, _r, row, itdoc)) {
 			LM_ERR("failed to convert bson at pos %d\n", row);
 			return -1;
 		}
@@ -739,7 +745,7 @@ static int db_mongodb_store_result(const db1_con_t* _h, db1_res_t** _r)
 		LM_ERR("failed to set the columns\n");
 		goto error;
 	}
-	if(db_mongdob_convert_result(_h, *_r)<0) {
+	if(db_mongodb_convert_result(_h, *_r)<0) {
 		LM_ERR("failed to set the rows in result\n");
 		goto error;
 	}
@@ -1204,7 +1210,7 @@ int db_mongodb_update(const db1_con_t* _h, const db_key_t* _k,
 		if(db_mongodb_bson_add(sdoc, _uk[i], _uv+i, i)<0)
 			goto error;
 	}
-	if(bson_append_document(udoc, "$set", 4, sdoc)<0) {
+	if(!bson_append_document(udoc, "$set", 4, sdoc)) {
                 LM_ERR("failed to append document to bson document\n");
                 goto error;
         }
