@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * dialog module - basic support for dialog tracking
  *
  * Copyright (C) 2006 Voice Sistem SRL
@@ -22,20 +20,13 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * History:
- * --------
- *  2006-04-14 initial version (bogdan)
- *  2006-11-28 Added statistic support for the number of early and failed
- *              dialogs. (Jeffrey Magder - SOMA Networks) 
- *  2007-04-30 added dialog matching without DID (dialog ID), but based only
- *              on RFC3261 elements - based on an original patch submitted 
- *              by Michel Bensoussan <michel@extricom.com> (bogdan)
- *  2007-05-15 added saving dialogs' information to database (ancuta)
- *  2007-07-04 added saving dialog cseq, contact, record route 
- *              and bind_addresses(sock_info) for caller and callee (ancuta)
- *  2008-04-14 added new type of callback to be triggered when dialogs are 
- *              loaded from DB (bogdan)
- *  2010-06-16 added sip-router rpc interface (osas)
+ */
+
+/*!
+ * \file
+ * \brief Module interface
+ * \ingroup dialog
+ * Module: \ref dialog
  */
 
 /**
@@ -75,6 +66,7 @@
 #include "../../parser/parse_to.h"
 #include "../../modules/tm/tm_load.h"
 #include "../../rpc_lookup.h"
+#include "../../srapi.h"
 #include "../rr/api.h"
 #include "dlg_hash.h"
 #include "dlg_timer.h"
@@ -108,6 +100,7 @@ static char* profiles_wv_s = NULL;
 static char* profiles_nv_s = NULL;
 str dlg_extra_hdrs = {NULL,0};
 static int db_fetch_rows = 200;
+static int db_skip_load = 0;
 int initial_cbs_inscript = 1;
 int dlg_wait_ack = 1;
 static int dlg_timer_procs = 0;
@@ -147,6 +140,8 @@ int dlg_ka_timer = 0;
 int dlg_ka_interval = 0;
 int dlg_clean_timer = 90;
 
+str dlg_lreq_callee_headers = {0};
+
 /* db stuff */
 static str db_url = str_init(DEFAULT_DB_URL);
 static unsigned int db_update_period = DB_DEFAULT_UPDATE_PERIOD;
@@ -297,6 +292,8 @@ static param_export_t mod_params[]={
 	{ "timeout_noreset",       INT_PARAM, &dlg_timeout_noreset      },
 	{ "timer_procs",           PARAM_INT, &dlg_timer_procs          },
 	{ "track_cseq_updates",    PARAM_INT, &_dlg_track_cseq_updates  },
+	{ "lreq_callee_headers",   PARAM_STR, &dlg_lreq_callee_headers  },
+	{ "db_skip_load",          INT_PARAM, &db_skip_load             },
 	{ 0,0,0 }
 };
 
@@ -373,7 +370,7 @@ static int fixup_profile(void** param, int param_no)
 	if (param_no==1) {
 		profile = search_dlg_profile( &s );
 		if (profile==NULL) {
-			LM_CRIT("profile <%s> not definited\n",s.s);
+			LM_CRIT("profile <%s> not defined\n",s.s);
 			return E_CFG;
 		}
 		pkg_free(*param);
@@ -463,6 +460,7 @@ static int pv_get_dlg_count(struct sip_msg *msg, pv_param_t *param,
 static int mod_init(void)
 {
 	unsigned int n;
+	sr_cfgenv_t *cenv = NULL;
 
 	if(dlg_ka_interval!=0 && dlg_ka_interval<30) {
 		LM_ERR("ka interval too low (%d), has to be at least 30\n",
@@ -500,10 +498,7 @@ static int mod_init(void)
 		return -1;
 
 	/* param checkings */
-	if (dlg_flag==-1) {
-		LM_ERR("no dlg flag set!!\n");
-		return -1;
-	} else if (dlg_flag>MAX_FLAG) {
+	if (dlg_flag>MAX_FLAG) {
 		LM_ERR("invalid dlg flag %d!!\n",dlg_flag);
 		return -1;
 	}
@@ -517,7 +512,7 @@ static int mod_init(void)
 	}
 
 	if (timeout_spec.s) {
-		if ( pv_parse_spec(&timeout_spec, &timeout_avp)==0 
+		if ( pv_parse_spec(&timeout_spec, &timeout_avp)==0
 				&& (timeout_avp.type!=PVT_AVP)){
 			LM_ERR("malformed or non AVP timeout "
 				"AVP definition in '%.*s'\n", timeout_spec.len,timeout_spec.s);
@@ -693,7 +688,7 @@ static int mod_init(void)
 			LM_ERR("db_url not configured for db_mode %d\n", dlg_db_mode);
 			return -1;
 		}
-		if (init_dlg_db(&db_url, dlg_hash_size, db_update_period,db_fetch_rows)!=0) {
+		if (init_dlg_db(&db_url, dlg_hash_size, db_update_period, db_fetch_rows, db_skip_load)!=0) {
 			LM_ERR("failed to initialize the DB support\n");
 			return -1;
 		}
@@ -709,8 +704,11 @@ static int mod_init(void)
 	/* timer process to clean old unconfirmed dialogs */
 	register_sync_timers(1);
 
-	if(_dlg_track_cseq_updates!=0)
+	if(_dlg_track_cseq_updates!=0) {
+		cenv = sr_cfgenv_get();
+		cenv->cb_cseq_update = dlg_cseq_update;
 		dlg_register_cseq_callbacks();
+	}
 
 	return 0;
 }
@@ -750,7 +748,7 @@ static int child_init(int rank)
 	}
 
 	if ( ((dlg_db_mode==DB_MODE_REALTIME || dlg_db_mode==DB_MODE_DELAYED) &&
-	(rank>0 || rank==PROC_TIMER)) ||
+	(rank>0 || rank==PROC_TIMER || rank==PROC_RPC)) ||
 	(dlg_db_mode==DB_MODE_SHUTDOWN && (rank==PROC_MAIN)) ) {
 		if ( dlg_connect_db(&db_url) ) {
 			LM_ERR("failed to connect to database (rank=%d)\n",rank);
@@ -1144,8 +1142,16 @@ static int w_dlg_set_timeout(struct sip_msg *msg, char *pto, char *phe, char *ph
 		LM_ERR("no timeout value\n");
 		return -1;
 	}
+	if(to<=0) {
+		LM_ERR("invalid timeout value: %d\n", to);
+		return -1;
+	}
 	if(phe!=NULL)
 	{
+		if(phi==NULL) {
+			LM_ERR("invalid number of parameters\n");
+			return -1;
+		}
 		if(fixup_get_ivalue(msg, (gparam_p)phe, (int*)&he)!=0)
 		{
 			LM_ERR("no hash entry value value\n");
@@ -1167,7 +1173,7 @@ static int w_dlg_set_timeout(struct sip_msg *msg, char *pto, char *phe, char *ph
 		return -1;
 	}
 
-	if(update_dlg_timeout(dlg, to) != 0) 
+	if(update_dlg_timeout(dlg, to) != 0)
 		return -1;
 
 	return 1;
@@ -1678,8 +1684,8 @@ static void internal_rpc_profile_print_dlgs(rpc_t *rpc, void *c, str *profile_na
 				ph=ph->next;
 			}while(ph!=profile->entries[i].first);
 		}
-		lock_release(&profile->lock);
 	}
+	lock_release(&profile->lock);
 }
 
 /*
