@@ -17,10 +17,6 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * History:
- * --------
- * 2008-04-20  initial version (bogdan)
- *
  */
 
 
@@ -398,6 +394,7 @@ int remove_profile(dlg_profile_table_t *profile, str *value, str *puid)
 				lh->next = lh->prev = NULL;
 				if(lh->linker) shm_free(lh->linker);
 				p_entry->content--;
+				lock_release(&profile->lock );
 				return 1;
 			}
 			lh = lh->next;
@@ -440,6 +437,10 @@ int profile_cleanup( struct sip_msg *msg, unsigned int flags, void *param )
 {
 	dlg_cell_t *dlg;
 
+	if(get_route_type()==LOCAL_ROUTE) {
+		return 1;
+	}
+
 	current_dlg_msg_id = 0;
 	current_dlg_msg_pid = 0;
 	dlg = dlg_get_ctx_dialog();
@@ -1281,9 +1282,9 @@ int dlg_json_to_profiles(dlg_cell_t *dlg, srjson_doc_t *jdoc)
 					puid.s = jt->valuestring;
 					puid.len = strlen(puid.s);
 				} else if(strcmp(jt->string, "expires")==0) {
-					expires = (time_t)jt->valueint;
+					expires = (time_t)SRJSON_GET_ULONG(jt);
 				} else if(strcmp(jt->string, "flags")==0) {
-					flags = jt->valueint;
+					flags = SRJSON_GET_UINT(jt);
 				}
 			}
 			if(name.s==NULL)
