@@ -50,9 +50,9 @@ extern int _dmq_usrloc_batch_usleep;
 
 static int add_contact(str aor, ucontact_info_t* ci)
 {
-	urecord_t* r;
+	urecord_t* r = NULL;
 	udomain_t* _d;
-	ucontact_t* c;
+	ucontact_t* c = NULL;
 	str contact;
 	int res;
 
@@ -61,37 +61,53 @@ static int add_contact(str aor, ucontact_info_t* ci)
 		return -1;
 	}
 
-	dmq_ul.lock_udomain(_d, &aor);
+	LM_DBG("aor: %.*s\n", aor.len, aor.s);
+	LM_DBG("ci->ruid: %.*s\n", ci->ruid.len, ci->ruid.s);
+	LM_DBG("aorhash: %i\n", dmq_ul.get_aorhash(&aor));
 
-	res = dmq_ul.get_urecord(_d, &aor, &r);
-	if (res < 0) {
-		LM_ERR("failed to retrieve record from usrloc\n");
-		goto error;
-	} else if ( res == 0) {
-		LM_DBG("'%.*s' found in usrloc\n", aor.len, ZSW(aor.s));
-		res = dmq_ul.get_ucontact(r, ci->c, ci->callid, ci->path, ci->cseq, &c);
-		LM_DBG("get_ucontact = %d\n", res);
-		if (res==-1) {
-			LM_ERR("Invalid cseq\n");
+	if (ci->ruid.len > 0) {
+		// Search by ruid, if possible
+		res = dmq_ul.get_urecord_by_ruid(_d, dmq_ul.get_aorhash(&aor), &ci->ruid, &r, &c);
+		if (res == 0) {
+			LM_DBG("Found contact\n");
+			dmq_ul.update_ucontact(r, c, ci);
+			LM_DBG("Release record\n");
+			dmq_ul.release_urecord(r);
+			LM_DBG("Unlock udomain\n");
+			dmq_ul.unlock_udomain(_d, &aor);
+			return 0;
+		}
+	}
+		dmq_ul.lock_udomain(_d, &aor);
+		res = dmq_ul.get_urecord(_d, &aor, &r);
+		if (res < 0) {
+			LM_ERR("failed to retrieve record from usrloc\n");
 			goto error;
-		} else if (res > 0 ) {
-			LM_DBG("Not found contact\n");
+		} else if ( res == 0) {
+			LM_DBG("'%.*s' found in usrloc\n", aor.len, ZSW(aor.s));
+			res = dmq_ul.get_ucontact(r, ci->c, ci->callid, ci->path, ci->cseq, &c);
+			LM_DBG("get_ucontact = %d\n", res);
+			if (res==-1) {
+				LM_ERR("Invalid cseq\n");
+				goto error;
+			} else if (res > 0 ) {
+				LM_DBG("Not found contact\n");
+				contact.s = ci->c->s;
+				contact.len = ci->c->len;
+				dmq_ul.insert_ucontact(r, &contact, ci, &c);
+			} else if (res == 0) {
+				LM_DBG("Found contact\n");
+				dmq_ul.update_ucontact(r, c, ci);
+			}
+		} else {
+			LM_DBG("'%.*s' Not found in usrloc\n", aor.len, ZSW(aor.s));
+			dmq_ul.insert_urecord(_d, &aor, &r);
+			LM_DBG("Insert record\n");
 			contact.s = ci->c->s;
 			contact.len = ci->c->len;
 			dmq_ul.insert_ucontact(r, &contact, ci, &c);
-		} else if (res == 0) {
-			LM_DBG("Found contact\n");
-			dmq_ul.update_ucontact(r, c, ci);
+			LM_DBG("Insert ucontact\n");
 		}
-	} else {
-		LM_DBG("'%.*s' Not found in usrloc\n", aor.len, ZSW(aor.s));
-		dmq_ul.insert_urecord(_d, &aor, &r);
-		LM_DBG("Insert record\n");
-		contact.s = ci->c->s;
-		contact.len = ci->c->len;
-		dmq_ul.insert_ucontact(r, &contact, ci, &c);
-		LM_DBG("Insert ucontact\n");
-	}
 
 		LM_DBG("Release record\n");
 		dmq_ul.release_urecord(r);
@@ -298,9 +314,13 @@ int usrloc_dmq_handle_msg(struct sip_msg* msg, peer_reponse_t* resp, dmq_node_t*
 	srjson_t *it = NULL;
 	static ucontact_info_t ci;
 
-	int action, expires, cseq, flags, cflags, q, last_modified, methods, reg_id;
+	unsigned int action, expires, cseq, flags, cflags, q, last_modified,
+				 methods, reg_id;
 	str aor, ruid, c, received, path, callid, user_agent, instance;
 
+	action = expires = cseq = flags = cflags = q = last_modified
+		= methods = reg_id = 0;
+
 	parse_from_header(msg);
 	body = ((struct to_body*)msg->from->parsed)->uri;
 
@@ -340,7 +360,7 @@ int usrloc_dmq_handle_msg(struct sip_msg* msg, peer_reponse_t* resp, dmq_node_t*
 		if (it->string == NULL) continue;
 
 		if (strcmp(it->string, "action")==0) {
-			action = it->valueint;
+			action = SRJSON_GET_UINT(it);
 		} else if (strcmp(it->string, "aor")==0) {
 			aor.s = it->valuestring;
 			aor.len = strlen(aor.s);
@@ -366,21 +386,21 @@ int usrloc_dmq_handle_msg(struct sip_msg* msg, peer_reponse_t* resp, dmq_node_t*
 			instance.s = it->valuestring;
 			instance.len = strlen(instance.s);
 		} else if (strcmp(it->string, "expires")==0) { //
-			expires = it->valueint;
+			expires = SRJSON_GET_UINT(it);
 		} else if (strcmp(it->string, "cseq")==0) {
-			cseq = it->valueint;
+			cseq = SRJSON_GET_UINT(it);
 		} else if (strcmp(it->string, "flags")==0) {
-			flags = it->valueint;
+			flags = SRJSON_GET_UINT(it);
 		} else if (strcmp(it->string, "cflags")==0) {
-			cflags = it->valueint;
+			cflags = SRJSON_GET_UINT(it);
 		} else if (strcmp(it->string, "q")==0) {
-			q = it->valueint;
+			q = SRJSON_GET_UINT(it);
 		} else if (strcmp(it->string, "last_modified")==0) {
-			last_modified = it->valueint;
+			last_modified = SRJSON_GET_UINT(it);
 		} else if (strcmp(it->string, "methods")==0) {
-			methods = it->valueint;
+			methods = SRJSON_GET_UINT(it);
 		} else if (strcmp(it->string, "reg_id")==0) {
-			reg_id = it->valueint;
+			reg_id = SRJSON_GET_UINT(it);
 		} else {
 			LM_ERR("unrecognized field in json object\n");
 		}
