@@ -40,6 +40,7 @@
 #ifdef WITH_XAVP
 #include "pv_xavp.h"
 #endif
+#include "pv_api.h"
 
 MODULE_VERSION
 
@@ -123,6 +124,18 @@ static pv_export_t mod_pvs[] = {
 	{{"aa", (sizeof("aa")-1)}, /* auth algorithm */
 		PVT_OTHER, pv_get_authattr, 0,
 		0, 0, pv_init_iname, 6},	
+	{{"adn", (sizeof("adn")-1)}, /* auth nonce */
+		PVT_OTHER, pv_get_authattr, 0,
+		0, 0, pv_init_iname, 7},	
+	{{"adc", (sizeof("adc")-1)}, /* auth cnonce */
+		PVT_OTHER, pv_get_authattr, 0,
+		0, 0, pv_init_iname, 8},	
+	{{"adr", (sizeof("adr")-1)}, /* auth response */
+		PVT_OTHER, pv_get_authattr, 0,
+		0, 0, pv_init_iname, 9},	
+	{{"ado", (sizeof("ado")-1)}, /* auth opaque */
+		PVT_OTHER, pv_get_authattr, 0,
+		0, 0, pv_init_iname, 10},	
 	{{"Au", (sizeof("Au")-1)}, /* */
 		PVT_OTHER, pv_get_acc_username, 0,
 		0, 0, pv_init_iname, 1},
@@ -438,9 +451,13 @@ static pv_export_t mod_pvs[] = {
 
 	{ {"shv", (sizeof("shv")-1)}, PVT_OTHER, pv_get_shvar,
 		pv_set_shvar, pv_parse_shvar_name, 0, 0, 0},
-	{ {"time", (sizeof("time")-1)}, PVT_CONTEXT, pv_get_time,
+	{ {"time", (sizeof("time")-1)}, PVT_CONTEXT, pv_get_local_time,
 		0, pv_parse_time_name, 0, 0, 0},
-	{ {"timef", (sizeof("timef")-1)}, PVT_CONTEXT, pv_get_strftime,
+	{ {"timef", (sizeof("timef")-1)}, PVT_CONTEXT, pv_get_local_strftime,
+		0, pv_parse_strftime_name, 0, 0, 0},
+	{ {"utime", (sizeof("utime")-1)}, PVT_CONTEXT, pv_get_utc_time,
+		0, pv_parse_time_name, 0, 0, 0},
+	{ {"utimef", (sizeof("utimef")-1)}, PVT_CONTEXT, pv_get_utc_strftime,
 		0, pv_parse_strftime_name, 0, 0, 0},
 	{ {"TV", (sizeof("TV")-1)}, PVT_OTHER, pv_get_timeval,
 		0, pv_parse_timeval_name, 0, 0, 0},
@@ -482,8 +499,11 @@ static int w_xavp_params_explode(sip_msg_t *msg, char *pparams, char *pxname);
 static int w_sbranch_set_ruri(sip_msg_t *msg, char p1, char *p2);
 static int w_sbranch_append(sip_msg_t *msg, char p1, char *p2);
 static int w_sbranch_reset(sip_msg_t *msg, char p1, char *p2);
+static int w_var_to_xavp(sip_msg_t *msg, char *p1, char *p2);
+static int w_xavp_to_var(sip_msg_t *msg, char *p1);
 
 static int pv_init_rpc(void);
+int pv_register_api(pv_api_t*);
 
 static cmd_export_t cmds[]={
 	{"pv_isset",  (cmd_function)pv_isset,  1, fixup_pvar_null, 0, 
@@ -493,6 +513,10 @@ static cmd_export_t cmds[]={
 #ifdef WITH_XAVP
 	{"pv_xavp_print",  (cmd_function)pv_xavp_print,  0, 0, 0, 
 		ANY_ROUTE },
+	{"pv_var_to_xavp",  (cmd_function)w_var_to_xavp, 2, 0, 0,
+		ANY_ROUTE },
+	{"pv_xavp_to_var",  (cmd_function)w_xavp_to_var, 1, 0, 0,
+		ANY_ROUTE },
 #endif
 	{"is_int", (cmd_function)is_int, 1, fixup_pvar_null, fixup_free_pvar_null,
 		ANY_ROUTE},
@@ -511,7 +535,8 @@ static cmd_export_t cmds[]={
 		ANY_ROUTE },
 	{"sbranch_reset",     (cmd_function)w_sbranch_reset,     0, 0, 0,
 		ANY_ROUTE },
-
+	/* API exports */
+	{"pv_register_api",   (cmd_function)pv_register_api,     NO_SCRIPT, 0, 0},
 	{0,0,0,0,0,0}
 };
 
@@ -544,7 +569,7 @@ static int mod_init(void)
 	{
 		LM_ERR("failed to register RPC commands\n");
 		return -1;
-    }
+	}
 	pv_init_sbranch();
 
 	return 0;
@@ -587,7 +612,10 @@ static int pv_unset(struct sip_msg* msg, char* pvid, char *foo)
 	pv_spec_t *sp;
 	
 	sp = (pv_spec_t*)pvid;
-	pv_set_spec_value(msg, sp, 0, NULL);
+	if(pv_set_spec_value(msg, sp, 0, NULL)<0) {
+		LM_ERR("faile to unset variable\n");
+		return -1;
+	}
 
 	return 1;
 }
@@ -681,6 +709,33 @@ static int is_int(struct sip_msg* msg, char* pvar, char* s2)
 	return -1;
 }
 
+static int w_var_to_xavp(sip_msg_t *msg, char *s1, char *s2)
+{
+	str xname, varname;
+
+	if(s1 == NULL || s2 == NULL) {
+		LM_ERR("wrong parameters\n");
+		return -1;
+	}
+
+	varname.len = strlen(s1); varname.s = s1;
+	xname.s = s2; xname.len = strlen(s2);
+	return pv_var_to_xavp(&varname, &xname);
+}
+
+static int w_xavp_to_var(sip_msg_t *msg, char *s1)
+{
+	str xname;
+
+	if(s1 == NULL) {
+		LM_ERR("wrong parameters\n");
+		return -1;
+	}
+
+	xname.s = s1; xname.len = strlen(s1);
+	return pv_xavp_to_var(&xname);
+}
+
 /**
  *
  */
