@@ -97,6 +97,8 @@ MODULE_VERSION
 #define	NAT_UAC_TEST_S_1918			0x08
 #define	NAT_UAC_TEST_RPORT			0x10
 
+#define COOKIE_SIZE					128
+#define HOSTNAME_SIZE				100
 
 #define DEFAULT_RTPP_SET_ID			0
 #define MAX_RTPP_TRIED_NODES			50
@@ -192,7 +194,7 @@ static struct rtpp_node *select_rtpp_node_new(str, str, int, struct rtpp_node **
 static struct rtpp_node *select_rtpp_node_old(str, str, int, enum rtpe_operation);
 static struct rtpp_node *select_rtpp_node(str, str, int, struct rtpp_node **, int, enum rtpe_operation);
 static int is_queried_node(struct rtpp_node *, struct rtpp_node **, int);
-static int build_rtpp_socks(unsigned int current_rtpp_no);
+static int build_rtpp_socks();
 static char *send_rtpp_command(struct rtpp_node *, bencode_item_t *, int *);
 static int get_extra_id(struct sip_msg* msg, str *id_str);
 
@@ -463,6 +465,7 @@ static int get_ip_type(char *str_addr)
 		LM_DBG("%s is an ipv6 addinfos\n", str_addr);
 	} else {
 		LM_DBG("%s is an unknown addinfos format AF=%d\n",str_addr, info->ai_family);
+		freeaddrinfo(info);
 		return -1;
 	}
 
@@ -590,8 +593,7 @@ static inline int str_eq(const str *p, const char *q) {
 }
 
 static inline str str_prefix(const str *p, const char *q) {
-	str ret;
-	ret.s = NULL;
+	str ret = STR_NULL;
 	int l = strlen(q);
 	if (p->len < l)
 		return ret;
@@ -1066,6 +1068,9 @@ static struct mi_root* mi_enable_rtp_proxy(struct mi_root *cmd_tree, void *param
 	str rtpp_url;
 	str snode, sattr, svalue;
 
+	if (build_rtpp_socks())
+		return init_mi_tree(400, MI_DB_ERR, MI_DB_ERR_LEN);
+
 	found = MI_FOUND_NONE;
 	found_rtpp_disabled = 0;
 	found_rtpp = NULL;
@@ -1410,6 +1415,9 @@ static struct mi_root* mi_ping_rtp_proxy(struct mi_root* cmd_tree, void* param)
 	str rtpp_url;
 	str snode, sattr, svalue;
 
+	if (build_rtpp_socks())
+		return init_mi_tree(400, MI_DB_ERR, MI_DB_ERR_LEN);
+
 	found = 0;
 	found_rtpp_disabled = 0;
 	found_rtpp = NULL;
@@ -1574,7 +1582,6 @@ static struct mi_root*
 mi_reload_rtp_proxy(struct mi_root* cmd_tree, void* param)
 {
 	struct mi_root *root = NULL;
-	unsigned int current_rtpp_no;
 
 	if (rtpp_db_url.s == NULL) {
 		// no database
@@ -1592,13 +1599,7 @@ mi_reload_rtp_proxy(struct mi_root* cmd_tree, void* param)
 				return 0;
 			}
 		} else {
-			lock_get(rtpp_no_lock);
-			current_rtpp_no = *rtpp_no;
-			lock_release(rtpp_no_lock);
-
-			if (rtpp_socks_size != current_rtpp_no) {
-				build_rtpp_socks(current_rtpp_no);
-			}
+			build_rtpp_socks();
 
 			// success reloading from database
 			root = init_mi_tree(200, MI_DB_OK, MI_DB_OK_LEN);
@@ -1793,20 +1794,29 @@ mod_init(void)
 	return 0;
 }
 
-static int build_rtpp_socks(unsigned int current_rtpp_no) {
+static int build_rtpp_socks() {
 	int n, i;
 	char *cp;
 	struct addrinfo hints, *res;
 	struct rtpp_set  *rtpp_list;
 	struct rtpp_node *pnode;
+	unsigned int current_rtpp_no;
 #ifdef IP_MTU_DISCOVER
 	int ip_mtu_discover = IP_PMTUDISC_DONT;
 #endif
 
+	lock_get(rtpp_no_lock);
+	current_rtpp_no = *rtpp_no;
+	lock_release(rtpp_no_lock);
+
+	if (current_rtpp_no == rtpp_socks_size)
+		return 0;
+
 	// close current sockets
 	for (i = 0; i < rtpp_socks_size; i++) {
 		if (rtpp_socks[i] >= 0) {
 			close(rtpp_socks[i]);
+			rtpp_socks[i] = -1;
 		}
 	}
 
@@ -1816,6 +1826,7 @@ static int build_rtpp_socks(unsigned int current_rtpp_no) {
 		LM_ERR("no more pkg memory for rtpp_socks\n");
 		return -1;
 	}
+	memset(rtpp_socks, -1, sizeof(int)*(rtpp_socks_size));
 
 	lock_get(rtpp_set_list->rset_head_lock);
 	for (rtpp_list = rtpp_set_list->rset_first; rtpp_list != 0;
@@ -1909,17 +1920,13 @@ child_init(int rank)
 	if(!rtpp_set_list)
 		return 0;
 
-	mypid = getpid();
-
-	lock_get(rtpp_no_lock);
-	rtpp_socks_size = *rtpp_no;
-	lock_release(rtpp_no_lock);
-
-	rtpp_socks = (int*)pkg_malloc(sizeof(int)*(rtpp_socks_size));
-	if (!rtpp_socks) {
-		return -1;
+	/* do not init sockets for PROC_INIT and main process when fork=yes */
+	if(rank==PROC_INIT || (rank==PROC_MAIN && dont_fork==0)) {
+		return 0;
 	}
 
+	mypid = getpid();
+
 	// vector of pointers to queried nodes
 	queried_nodes_ptr = (struct rtpp_node**)pkg_malloc(queried_nodes_limit * sizeof(struct rtpp_node*));
 	if (!queried_nodes_ptr) {
@@ -1929,9 +1936,8 @@ child_init(int rank)
 	memset(queried_nodes_ptr, 0, queried_nodes_limit * sizeof(struct rtpp_node*));
 
 	/* Iterate known RTP proxies - create sockets */
-	if (rtpp_socks_size) {
-		build_rtpp_socks(rtpp_socks_size);
-	}
+	if (build_rtpp_socks())
+		return -1;
 
 	return 0;
 }
@@ -2016,12 +2022,11 @@ static void mod_destroy(void)
 }
 
 
-
 static char * gencookie(void)
 {
 	static char cook[34];
 
-	sprintf(cook, "%d_%u ", (int)mypid, myseqn);
+	snprintf(cook, 34, "%d_%d_%u ", server_id, (int)mypid, myseqn);
 	myseqn++;
 	return cook;
 }
@@ -2835,15 +2840,8 @@ static struct rtpp_node *
 select_rtpp_node(str callid, str viabranch, int do_test, struct rtpp_node **queried_nodes_ptr, int queried_nodes, enum rtpe_operation op)
 {
 	struct rtpp_node *node = NULL;
-	unsigned int current_rtpp_no;
 
-	lock_get(rtpp_no_lock);
-	current_rtpp_no = *rtpp_no;
-	lock_release(rtpp_no_lock);
-
-	if (rtpp_socks_size != current_rtpp_no) {
-		build_rtpp_socks(current_rtpp_no);
-	}
+	build_rtpp_socks();
 
 	if (!active_rtpp_set) {
 		default_rtpp_set = select_rtpp_set(setid_default);
