@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * hep related functions
  *
  * Copyright (C) 2011-14 Alexandr Dubovikov <alexandr.dubovikov@gmail.com>
@@ -37,15 +35,16 @@
 #include "sipcapture.h"
 
 
-static int show_error = 0;
+
 static int count = 0;
 
-struct hep_timehdr* heptime;
+struct hep_timeinfo* heptime;
 
 /* HEPv2 HEPv3 */
 int hepv2_received(char *buf, unsigned int len, struct receive_info *ri);
 int hepv3_received(char *buf, unsigned int len, struct receive_info *ri);
 int parsing_hepv3_message(char *buf, unsigned int len);
+
 /**
  * HEP message
  */
@@ -58,10 +57,7 @@ int hep_msg_received(void *data)
         struct receive_info *ri;
 
         if(!hep_capture_on) {
-        	if(show_error == 0) {
-                	LOG(L_ERR, "sipcapture:hep_msg_received HEP is not enabled\n");
-                	show_error = 1;
-        	}
+                LOG(L_ERR, "sipcapture:hep_msg_received HEP is not enabled\n");
                 return -1;
         }
 
@@ -71,6 +67,9 @@ int hep_msg_received(void *data)
         len = (unsigned *)srevp[1];
         ri = (struct receive_info *)srevp[2];                        
 
+	correlation_id = NULL;
+	authkey = NULL;
+
 	count++;
         struct hep_hdr *heph;
         /* hep_hdr */
@@ -102,9 +101,12 @@ int hepv2_received(char *buf, unsigned int len, struct receive_info *ri){
         struct hep_iphdr *hepiph = NULL;
 
 	struct hep_timehdr* heptime_tmp = NULL;
-        memset(heptime, 0, sizeof(struct hep_timehdr));
+        memset(heptime, 0, sizeof(struct hep_timeinfo));
 
         struct hep_ip6hdr *hepip6h = NULL;
+            	        
+	correlation_id = NULL;
+	authkey = NULL;
 
 	hep_offset = 0; 
 	
@@ -138,7 +140,7 @@ int hepv2_received(char *buf, unsigned int len, struct receive_info *ri){
         else if(heph->hp_p == IPPROTO_SCTP) ri->proto=PROTO_SCTP;
 #endif
         else {
-        	LOG(L_ERR, "ERROR: sipcapture:hep_msg_received: unknow protocol [%d]\n",heph->hp_p);
+        	LOG(L_ERR, "ERROR: sipcapture:hep_msg_received: unknown protocol [%d]\n",heph->hp_p);
                 ri->proto = PROTO_NONE;
 	}
 
@@ -261,7 +263,7 @@ int parsing_hepv3_message(char *buf, unsigned int len) {
 	memset(hg, 0, sizeof(struct hep_generic_recv));
 
 	
-        memset(heptime, 0, sizeof(struct hep_timehdr));	
+        memset(heptime, 0, sizeof(struct hep_timeinfo));	
 	        
 
 	/* HEADER */
@@ -276,6 +278,8 @@ int parsing_hepv3_message(char *buf, unsigned int len) {
         src_ip.af = 0;
                 	        
 	payload = NULL;
+	correlation_id = NULL;
+	authkey = NULL;
 
 	i = sizeof(hep_ctrl_t);	        
 	        
@@ -391,7 +395,7 @@ int parsing_hepv3_message(char *buf, unsigned int len) {
                                 case 12:
                                         hg->capt_id  = (hep_chunk_uint32_t *) (tmp);
                                         i+=chunk_length;
-                                        heptime->captid = ntohs(hg->capt_id->data);
+                                        heptime->captid = ntohl(hg->capt_id->data);
                                         totelem++;
                                         break;
 
@@ -483,13 +487,15 @@ int parsing_hepv3_message(char *buf, unsigned int len) {
 	/*TIME*/ 
         heptime->tv_sec = hg->time_sec->data;
         heptime->tv_usec = hg->time_usec->data;
-        heptime->captid = ntohs(hg->capt_id->data);
+        heptime->captid = ntohl(hg->capt_id->data);
           
 
         if(payload != NULL ) {
-                /* and now recieve message */                
+                /* and now recieve message */
                 if (hg->proto_t->data == 5) receive_logging_json_msg(payload, payload_len, hg, "rtcp_capture");
-                else if (hg->proto_t->data == 100) receive_logging_json_msg(payload, payload_len, hg, "logs_capture");                
+                else if (hg->proto_t->data == 32) receive_logging_json_msg(payload, payload_len, hg, "report_capture");
+                else if (hg->proto_t->data == 99) receive_logging_json_msg(payload, payload_len, hg, "report_capture");
+                else if (hg->proto_t->data == 100) receive_logging_json_msg(payload, payload_len, hg, "logs_capture");
                 else receive_msg(payload, payload_len, &ri);
         }
 
@@ -509,6 +515,3 @@ error:
 }
 
 
-
-
-
