@@ -1,6 +1,4 @@
 /* 
- * $Id$ 
- *
  * siptrace module - helper module to trace sip messages
  *
  * Copyright (C) 2006 Voice Sistem S.R.L.
@@ -52,6 +50,7 @@
 #include "../../modules/sl/sl.h"
 #include "../../str.h"
 #include "../../onsend.h"
+#include "../../events.h"
 
 #include "../../modules/sipcapture/hep.h"
 
@@ -72,6 +71,7 @@ struct _siptrace_data {
 	char *dir;
 	str fromtag;
 	str fromip;
+	str totag;
 	str toip;
 	char toip_buff[IP_ADDR_MAX_STR_SIZE+12];
 	char fromip_buff[IP_ADDR_MAX_STR_SIZE+12];
@@ -107,6 +107,10 @@ static void trace_sl_ack_in(sl_cbp_t *slcb);
 static int trace_send_hep_duplicate(str *body, str *from, str *to, struct dest_info*);
 static int pipport2su (char *pipport, union sockaddr_union *tmp_su, unsigned int *proto);
 
+int siptrace_net_data_recv(void *data);
+int siptrace_net_data_send(void *data);
+static int _siptrace_mode = 0;
+
 
 static struct mi_root* sip_trace_mi(struct mi_root* cmd, void* param );
 
@@ -123,8 +127,10 @@ static str toip_column        = str_init("toip");        /* 07 */
 static str fromtag_column     = str_init("fromtag");     /* 08 */
 static str direction_column   = str_init("direction");   /* 09 */
 static str time_us_column     = str_init("time_us");     /* 10 */
+static str totag_column       = str_init("totag");       /* 11 */
 
-#define NR_KEYS 11
+#define NR_KEYS 12
+#define SIP_TRACE_TABLE_VERSION 4
 
 #define XHEADERS_BUFSIZE 512
 
@@ -173,7 +179,7 @@ db_func_t db_funcs;      		/*!< Database functions */
  */
 static cmd_export_t cmds[] = {
 	{"sip_trace", (cmd_function)sip_trace, 0, 0, 0, ANY_ROUTE},
-    {"sip_trace", (cmd_function)sip_trace, 1, fixup_siptrace, 0, ANY_ROUTE},
+	{"sip_trace", (cmd_function)sip_trace, 1, fixup_siptrace, 0, ANY_ROUTE},
 	{0, 0, 0, 0, 0, 0}
 };
 
@@ -193,6 +199,7 @@ static param_export_t params[] = {
 	{"fromip_column",      PARAM_STR, &fromip_column      },
 	{"toip_column",        PARAM_STR, &toip_column        },
 	{"fromtag_column",     PARAM_STR, &fromtag_column     },
+	{"totag_column",       PARAM_STR, &totag_column       },
 	{"direction_column",   PARAM_STR, &direction_column   },
 	{"trace_flag",         INT_PARAM, &trace_flag           },
 	{"trace_on",           INT_PARAM, &trace_on             },
@@ -205,10 +212,11 @@ static param_export_t params[] = {
 	{"xheaders_write",     INT_PARAM, &xheaders_write       },
 	{"xheaders_read",      INT_PARAM, &xheaders_read        },
 	{"hep_mode_on",        INT_PARAM, &hep_mode_on          },	 
-    {"force_send_sock",    PARAM_STR, &force_send_sock_str	},
+	{"force_send_sock",    PARAM_STR, &force_send_sock_str	},
 	{"hep_version",        INT_PARAM, &hep_version          },
 	{"hep_capture_id",     INT_PARAM, &hep_capture_id       },	        
 	{"trace_delayed",      INT_PARAM, &trace_delayed        },
+	{"trace_mode",         PARAM_INT, &_siptrace_mode       },
 	{0, 0, 0}
 };
 
@@ -295,8 +303,8 @@ static int mod_init(void)
 	*trace_to_database_flag = trace_to_database;
 
 	if(hep_version != 1 && hep_version != 2) {
-	    LM_ERR("unsupported version of HEP");
-	    return -1;
+		LM_ERR("unsupported version of HEP");
+		return -1;
 	}
 
 	/* Find a database module if needed */
@@ -314,12 +322,11 @@ static int mod_init(void)
 		}
 	}
 
-        if(hep_version != 1 && hep_version != 2) {
-  
-                  LM_ERR("unsupported version of HEP");
-                  return -1;
-        }                                          
+	if(hep_version != 1 && hep_version != 2) {
 
+		LM_ERR("unsupported version of HEP");
+		return -1;
+	}
 
 	trace_on_flag = (int*)shm_malloc(sizeof(int));
 	if(trace_on_flag==NULL) {
@@ -389,19 +396,19 @@ static int mod_init(void)
 
 	if(force_send_sock_str.s!=0)
 	{
-	    force_send_sock_str.len = strlen(force_send_sock_str.s);
-	    force_send_sock_uri = (struct sip_uri *)pkg_malloc(sizeof(struct sip_uri));
-	    if(force_send_sock_uri==0)
-	    {
-	        LM_ERR("no more pkg memory left\n");
-	        return -1;
-	    }
-	    memset(force_send_sock_uri, 0, sizeof(struct sip_uri));
-	    if(parse_uri(force_send_sock_str.s, force_send_sock_str.len, force_send_sock_uri)<0)
-	    {
-	        LM_ERR("bad dup uri\n");
-	        return -1;
-	    }
+		force_send_sock_str.len = strlen(force_send_sock_str.s);
+		force_send_sock_uri = (struct sip_uri *)pkg_malloc(sizeof(struct sip_uri));
+		if(force_send_sock_uri==0)
+		{
+			LM_ERR("no more pkg memory left\n");
+			return -1;
+		}
+		memset(force_send_sock_uri, 0, sizeof(struct sip_uri));
+		if(parse_uri(force_send_sock_str.s, force_send_sock_str.len, force_send_sock_uri)<0)
+		{
+			LM_ERR("bad dup uri\n");
+			return -1;
+		}
 	}
 
 	if(traced_user_avp_str.s && traced_user_avp_str.len > 0)
@@ -447,6 +454,10 @@ static int mod_init(void)
 		trace_table_avp_type = 0;
 	}
 
+	if(_siptrace_mode==1) {
+		sr_event_register_cb(SREV_NET_DATA_RECV, siptrace_net_data_recv);
+		sr_event_register_cb(SREV_NET_DATA_SEND, siptrace_net_data_send);
+	}
 	return 0;
 }
 
@@ -463,6 +474,12 @@ static int child_init(int rank)
 			LM_ERR("unable to connect to database. Please check configuration.\n");
 			return -1;
 		}
+		if (db_check_table_version(&db_funcs, db_con, &siptrace_table,
+					SIP_TRACE_TABLE_VERSION) < 0) {
+			LM_ERR("error during table version check\n");
+			db_funcs.close(db_con);		
+			return -1;
+		}
 	}
 
 	return 0;
@@ -527,6 +544,11 @@ static int sip_trace_prepare(sip_msg_t *msg)
 		goto error;
 	}
 
+	if(parse_to_header(msg)==-1 || msg->to==NULL || get_to(msg)==NULL) {
+		LM_ERR("cannot parse To header\n");
+		goto error;
+	}
+
 	if(parse_headers(msg, HDR_CALLID_F, 0)!=0 || msg->callid==NULL
 			|| msg->callid->body.s==NULL) {
 		LM_ERR("cannot parse call-id\n");
@@ -739,7 +761,7 @@ static int sip_trace_store(struct _siptrace_data *sto, struct dest_info *dst)
 		return -1;
 
 	if(hep_mode_on) trace_send_hep_duplicate(&sto->body, &sto->fromip, &sto->toip, dst);
-    else trace_send_duplicate(sto->body.s, sto->body.len, dst);
+	else trace_send_duplicate(sto->body.s, sto->body.len, dst);
 
 	if (sip_trace_xheaders_free(sto) != 0)
 		return -1;
@@ -749,6 +771,11 @@ static int sip_trace_store(struct _siptrace_data *sto, struct dest_info *dst)
 
 static int sip_trace_store_db(struct _siptrace_data *sto)
 {
+	if(db_con==NULL) {
+		LM_DBG("database connection not initialized\n");
+		return -1;
+	}
+
 	if(trace_to_database_flag==NULL || *trace_to_database_flag==0)
 		goto done;
 
@@ -809,6 +836,11 @@ static int sip_trace_store_db(struct _siptrace_data *sto)
 	db_vals[10].nul = 0;
 	db_vals[10].val.int_val = sto->tv.tv_usec;
 
+	db_keys[11] = &totag_column;
+	db_vals[11].type = DB1_STR;
+	db_vals[11].nul = 0;
+	db_vals[11].val.str_val = sto->totag;
+
 	db_funcs.use_table(db_con, siptrace_get_table());
 
 	if(trace_on_flag!=NULL && *trace_on_flag!=0) {
@@ -938,14 +970,14 @@ static int sip_trace(struct sip_msg *msg, struct dest_info * dst, char *dir)
 	struct onsend_info *snd_inf = NULL;
 
 	if (dst){
-	    if (dst->send_sock == 0){
-	        dst->send_sock=get_send_socket(0, &dst->to, dst->proto);
-	        if (dst->send_sock==0){
-	            LM_ERR("can't forward to af %d, proto %d no corresponding"
-	                    " listening socket\n", dst->to.s.sa_family, dst->proto);
-	            return -1;
-	        }
-	    }
+		if (dst->send_sock == 0){
+			dst->send_sock=get_send_socket(0, &dst->to, dst->proto);
+			if (dst->send_sock==0){
+				LM_ERR("can't forward to af %d, proto %d no corresponding"
+						" listening socket\n", dst->to.s.sa_family, dst->proto);
+				return -1;
+			}
+		}
 	}
 
 	if(msg==NULL) {
@@ -990,29 +1022,41 @@ static int sip_trace(struct sip_msg *msg, struct dest_info * dst, char *dir)
 		sto.body.s = msg->buf;
 		sto.body.len = msg->len;
 
-		siptrace_copy_proto(msg->rcv.proto, sto.fromip_buff);
-		strcat(sto.fromip_buff, ip_addr2a(&msg->rcv.src_ip));
-		strcat(sto.fromip_buff,":");
-		strcat(sto.fromip_buff, int2str(msg->rcv.src_port, NULL));
-		sto.fromip.s = sto.fromip_buff;
-		sto.fromip.len = strlen(sto.fromip_buff);
-
-		siptrace_copy_proto(msg->rcv.proto, sto.toip_buff);
-		strcat(sto.toip_buff, ip_addr2a(&msg->rcv.dst_ip));
-		strcat(sto.toip_buff,":");
-		strcat(sto.toip_buff, int2str(msg->rcv.dst_port, NULL));
-		sto.toip.s = sto.toip_buff;
-		sto.toip.len = strlen(sto.toip_buff);
-
 		sto.dir = (dir)?dir:"in";
+
+		if (trace_local_ip.s && trace_local_ip.len > 0 && strncmp(sto.dir, "out", 3) == 0) {
+			sto.fromip = trace_local_ip;
+		} else {
+			siptrace_copy_proto(msg->rcv.proto, sto.fromip_buff);
+			strcat(sto.fromip_buff, ip_addr2a(&msg->rcv.src_ip));
+			strcat(sto.fromip_buff,":");
+			strcat(sto.fromip_buff, int2str(msg->rcv.src_port, NULL));
+			sto.fromip.s = sto.fromip_buff;
+			sto.fromip.len = strlen(sto.fromip_buff);
+		}
+
+		if (trace_local_ip.s && trace_local_ip.len > 0 && strncmp(sto.dir, "in", 2) == 0) {
+			sto.toip = trace_local_ip;
+		} else {
+			siptrace_copy_proto(msg->rcv.proto, sto.toip_buff);
+			strcat(sto.toip_buff, ip_addr2a(&msg->rcv.dst_ip));
+			strcat(sto.toip_buff,":");
+			strcat(sto.toip_buff, int2str(msg->rcv.dst_port, NULL));
+			sto.toip.s = sto.toip_buff;
+			sto.toip.len = strlen(sto.toip_buff);
+		}
 	} else {
 		sto.body.s   = snd_inf->buf;
 		sto.body.len = snd_inf->len;
 
-		strncpy(sto.fromip_buff, snd_inf->send_sock->sock_str.s,
-				snd_inf->send_sock->sock_str.len);
-		sto.fromip.s = sto.fromip_buff;
-		sto.fromip.len = strlen(sto.fromip_buff);
+		if (trace_local_ip.s && trace_local_ip.len > 0) {
+			sto.fromip = trace_local_ip;
+		} else {
+			strncpy(sto.fromip_buff, snd_inf->send_sock->sock_str.s,
+					snd_inf->send_sock->sock_str.len);
+			sto.fromip.s = sto.fromip_buff;
+			sto.fromip.len = strlen(sto.fromip_buff);
+		}
 
 		siptrace_copy_proto(snd_inf->send_sock->proto, sto.toip_buff);
 		strcat(sto.toip_buff, suip2a(snd_inf->to, sizeof(*snd_inf->to)));
@@ -1025,6 +1069,7 @@ static int sip_trace(struct sip_msg *msg, struct dest_info * dst, char *dir)
 	}
 
 	sto.fromtag = get_from(msg)->tag_value;
+	sto.totag = get_to(msg)->tag_value;
 
 #ifdef STATISTICS
 	if(msg->first_line.type==SIP_REPLY) {
@@ -1210,6 +1255,7 @@ static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps)
 	sto.dir = "out";
 
 	sto.fromtag = get_from(msg)->tag_value;
+	sto.totag = get_to(msg)->tag_value;
 
 #ifdef STATISTICS
 	sto.stat = siptrace_req;
@@ -1282,6 +1328,7 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 	sto.dir = "in";
 
 	sto.fromtag = get_from(msg)->tag_value;
+	sto.totag = get_to(msg)->tag_value;
 #ifdef STATISTICS
 	sto.stat = siptrace_rpl;
 #endif
@@ -1390,6 +1437,7 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 
 	sto.dir = "out";
 	sto.fromtag = get_from(msg)->tag_value;
+	sto.totag = get_to(msg)->tag_value;
 
 #ifdef STATISTICS
 	sto.stat = siptrace_rpl;
@@ -1476,6 +1524,7 @@ static void trace_sl_onreply_out(sl_cbp_t *slcbp)
 
 	sto.dir = "out";
 	sto.fromtag = get_from(msg)->tag_value;
+	sto.totag = get_to(msg)->tag_value;
 
 #ifdef STATISTICS
 	sto.stat = siptrace_rpl;
@@ -1545,34 +1594,34 @@ static int trace_send_duplicate(char *buf, int len, struct dest_info *dst2)
 		return 0;
 
 	init_dest_info(&dst);
-	/* create a temporary proxy*/
-	dst.proto = PROTO_UDP;
-	p=mk_proxy(&dup_uri->host, (dup_uri->port_no)?dup_uri->port_no:SIP_PORT,
-			dst.proto);
-	if (p==0)
-	{
-		LM_ERR("bad host name in uri\n");
-		return -1;
-	}
 
 	if (!dst2){
-	    init_dest_info(&dst);
-	    /* create a temporary proxy*/
-	    dst.proto = PROTO_UDP;
-	    p=mk_proxy(&dup_uri->host, (dup_uri->port_no)?dup_uri->port_no:SIP_PORT,
-	             dst.proto);
-	    if (p==0){
-	        LM_ERR("bad host name in uri\n");
-	        return -1;
-	    }
-	    hostent2su(&dst.to, &p->host, p->addr_idx, (p->port)?p->port:SIP_PORT);
+		/* create a temporary proxy from dst param */
+		dst.proto = PROTO_UDP;
+		p=mk_proxy(&dup_uri->host, (dup_uri->port_no)?dup_uri->port_no:SIP_PORT,
+				dst.proto);
+		if (p==0){
+			LM_ERR("bad host name in uri\n");
+			return -1;
+		}
+		hostent2su(&dst.to, &p->host, p->addr_idx, (p->port)?p->port:SIP_PORT);
 
-	    dst.send_sock=get_send_socket(0, &dst.to, dst.proto);
-	    if (dst.send_sock==0){
-	        LM_ERR("can't forward to af %d, proto %d no corresponding"
-	                " listening socket\n", dst.to.s.sa_family, dst.proto);
-	        goto error;
-	    }
+		dst.send_sock=get_send_socket(0, &dst.to, dst.proto);
+		if (dst.send_sock==0){
+			LM_ERR("can't forward to af %d, proto %d no corresponding"
+					" listening socket\n", dst.to.s.sa_family, dst.proto);
+			goto error;
+		}
+	} else {
+		/* create a temporary proxy to dup uri */
+		dst.proto = PROTO_UDP;
+		p=mk_proxy(&dup_uri->host, (dup_uri->port_no)?dup_uri->port_no:SIP_PORT,
+				dst.proto);
+		if (p==0)
+		{
+			LM_ERR("bad host name in uri\n");
+			return -1;
+		}
 	}
 
 	if (msg_send((dst2)?dst2:&dst, buf, len)<0)
@@ -1582,15 +1631,15 @@ static int trace_send_duplicate(char *buf, int len, struct dest_info *dst2)
 	}
 
 	if (p){
-	    free_proxy(p); /* frees only p content, not p itself */
-	    pkg_free(p);
+		free_proxy(p); /* frees only p content, not p itself */
+		pkg_free(p);
 	}
 	return 0;
 error:
-    if (p){
-	free_proxy(p); /* frees only p content, not p itself */
-	pkg_free(p);
-    }
+	if (p){
+		free_proxy(p); /* frees only p content, not p itself */
+		pkg_free(p);
+	}
 	return -1;
 }
 
@@ -1609,7 +1658,7 @@ static int trace_send_hep_duplicate(str *body, str *from, str *to, struct dest_i
 	struct hep_timehdr hep_time;
 	struct timeval tvb;
 	struct timezone tz;
-	                 
+
 	struct hep_ip6hdr hep_ip6header;
 
 	if(body->s==NULL || body->len <= 0)
@@ -1619,8 +1668,8 @@ static int trace_send_hep_duplicate(str *body, str *from, str *to, struct dest_i
 		return 0;
 
 
-        gettimeofday( &tvb, &tz );
-        
+	gettimeofday( &tvb, &tz );
+
 
 	/* message length */
 	len = body->len 
@@ -1639,50 +1688,50 @@ static int trace_send_hep_duplicate(str *body, str *from, str *to, struct dest_i
 
 	/* check if from and to are in the same family*/
 	if(from_su.s.sa_family != to_su.s.sa_family) {
-		LOG(L_ERR, "ERROR: trace_send_hep_duplicate: interworking detected ?\n");
+		LM_ERR("interworking detected ?\n");
 		goto error;
 	}
 
-    if (!dst2){
-	init_dest_info(&dst);
-	/* create a temporary proxy*/
-	dst.proto = PROTO_UDP;
-	p=mk_proxy(&dup_uri->host, (dup_uri->port_no)?dup_uri->port_no:SIP_PORT,
-			dst.proto);
-	if (p==0)
-	{
-		LM_ERR("bad host name in uri\n");
-		goto error;
+	if (!dst2){
+		init_dest_info(&dst);
+		/* create a temporary proxy*/
+		dst.proto = PROTO_UDP;
+		p=mk_proxy(&dup_uri->host, (dup_uri->port_no)?dup_uri->port_no:SIP_PORT,
+				dst.proto);
+		if (p==0)
+		{
+			LM_ERR("bad host name in uri\n");
+			goto error;
+		}
+
+		hostent2su(&dst.to, &p->host, p->addr_idx, (p->port)?p->port:SIP_PORT);
+		LM_DBG("setting up the socket_info\n");
+		dst_fin = &dst;
+	} else {
+		dst_fin = dst2;
 	}
 
-	hostent2su(&dst.to, &p->host, p->addr_idx, (p->port)?p->port:SIP_PORT);
-	LM_DBG("setting up the socket_info\n");
-	dst_fin = &dst;
-    } else {
-        dst_fin = dst2;
-    }
+	if (force_send_sock_str.s) {
+		LM_DBG("force_send_sock activated, grep for the sock_info\n");
+		si = grep_sock_info(&force_send_sock_uri->host,
+				(force_send_sock_uri->port_no)?force_send_sock_uri->port_no:SIP_PORT,
+				PROTO_UDP);
+		if (!si) {
+			LM_WARN("cannot grep socket info\n");
+		} else {
+			LM_DBG("found socket while grep: [%.*s] [%.*s]\n", si->name.len, si->name.s, si->address_str.len, si->address_str.s);
+			dst_fin->send_sock = si;
+		}
+	}
 
-    if (force_send_sock_str.s) {
-        LM_DBG("force_send_sock activated, grep for the sock_info\n");
-        si = grep_sock_info(&force_send_sock_uri->host,
-                (force_send_sock_uri->port_no)?force_send_sock_uri->port_no:SIP_PORT,
-                PROTO_UDP);
-        if (!si) {
-             LM_WARN("cannot grep socket info\n");
-        } else {
-            LM_DBG("found socket while grep: [%.*s] [%.*s]\n", si->name.len, si->name.s, si->address_str.len, si->address_str.s);
-            dst_fin->send_sock = si;
-        }
-    }
-
-    if (dst_fin->send_sock == 0) {
-        dst_fin->send_sock=get_send_socket(0, &dst_fin->to, dst_fin->proto);
-        if (dst_fin->send_sock == 0) {
-            LM_ERR("can't forward to af %d, proto %d no corresponding"
-                    " listening socket\n", dst_fin->to.s.sa_family, dst_fin->proto);
-            goto error;
-        }
-    }
+	if (dst_fin->send_sock == 0) {
+		dst_fin->send_sock=get_send_socket(0, &dst_fin->to, dst_fin->proto);
+		if (dst_fin->send_sock == 0) {
+			LM_ERR("can't forward to af %d, proto %d no corresponding"
+					" listening socket\n", dst_fin->to.s.sa_family, dst_fin->proto);
+			goto error;
+		}
+	}
 
 	/* Version && proto && length */
 	hdr.hp_l = sizeof(struct hep_hdr);
@@ -1716,7 +1765,7 @@ static int trace_send_hep_duplicate(str *body, str *from, str *to, struct dest_i
 		len = sizeof(struct hep_ip6hdr);
 	}
 	else {
-		LOG(L_ERR, "ERROR: trace_send_hep_duplicate: Unsupported protocol family\n");
+		LM_ERR("Unsupported protocol family\n");
 		goto error;;
 	}
 
@@ -1727,7 +1776,7 @@ static int trace_send_hep_duplicate(str *body, str *from, str *to, struct dest_i
 	len += sizeof(struct hep_hdr) + body->len;
 	buffer = (void *)pkg_malloc(len+1);
 	if (buffer==0){
-		LOG(L_ERR, "ERROR: trace_send_hep_duplicate: out of memory\n");
+		LM_ERR("out of memory\n");
 		goto error;
 	}
 
@@ -1750,28 +1799,28 @@ static int trace_send_hep_duplicate(str *body, str *from, str *to, struct dest_i
 
 	if(hep_version == 2) {
 
-                hep_time.tv_sec = tvb.tv_sec;
-                hep_time.tv_usec = tvb.tv_usec;
-                hep_time.captid = hep_capture_id;
+		hep_time.tv_sec = tvb.tv_sec;
+		hep_time.tv_usec = tvb.tv_usec;
+		hep_time.captid = hep_capture_id;
 
-                memcpy((void*)buffer+buflen, &hep_time, sizeof(struct hep_timehdr));
-                buflen += sizeof(struct hep_timehdr);
-        }
+		memcpy((void*)buffer+buflen, &hep_time, sizeof(struct hep_timehdr));
+		buflen += sizeof(struct hep_timehdr);
+	}
 
 	/* PAYLOAD */
 	memcpy((void*)(buffer + buflen) , (void*)body->s, body->len);
 	buflen +=body->len;
 
-	if (msg_send(dst_fin, buffer, buflen)<0)
+	if (msg_send_buffer(dst_fin, buffer, buflen, 1)<0)
 	{
 		LM_ERR("cannot send hep duplicate message\n");
 		goto error;
 	}
 
-    if (p) {
-	free_proxy(p); /* frees only p content, not p itself */
-	pkg_free(p);
-    }
+	if (p) {
+		free_proxy(p); /* frees only p content, not p itself */
+		pkg_free(p);
+	}
 	pkg_free(buffer);
 	return 0;
 error:
@@ -1815,14 +1864,14 @@ static int pipport2su (char *pipport, union sockaddr_union *tmp_su, unsigned int
 		LM_ERR("bad protocol %s\n", pipport);
 		return -1;
 	}
-	
+
 	if((len = strlen(pipport)) >= 256) {
 		LM_ERR("too big pipport\n");
 		goto error;
 	}
 
 	/* our tmp string */
-        strncpy(tmp_piport, pipport, len+1);
+	strncpy(tmp_piport, pipport, len+1);
 
 	len = 0;
 
@@ -1839,19 +1888,19 @@ static int pipport2su (char *pipport, union sockaddr_union *tmp_su, unsigned int
 		port_no = 0;
 	}
 	else {
-        	/*the address contains a port number*/
-        	*p = '\0';
-        	p++;
-        	port_str.s = p;
-        	port_str.len = strlen(p);
-        	LM_DBG("the port string is %s\n", p);
-        	if(str2int(&port_str, &port_no) != 0 ) {
-	        	LM_ERR("there is not a valid number port\n");
-	        	goto error;
-        	}
-        	*p = '\0';
-        }
-        
+		/*the address contains a port number*/
+		*p = '\0';
+		p++;
+		port_str.s = p;
+		port_str.len = strlen(p);
+		LM_DBG("the port string is %s\n", p);
+		if(str2int(&port_str, &port_no) != 0 ) {
+			LM_ERR("there is not a valid number port\n");
+			goto error;
+		}
+		*p = '\0';
+	}
+
 	/* now IPv6 address has no brakets. It should be fixed! */
 	if (host_s[0] == '[') {
 		len = strlen(host_s + 1) - 1;
@@ -1879,6 +1928,95 @@ error:
 	return -1;
 }
 
+
+/**
+ *
+ */
+int siptrace_net_data_recv(void *data)
+{
+	sr_net_info_t *nd;
+	struct _siptrace_data sto;
+
+	if(data==0)
+		return -1;
+
+	nd = (sr_net_info_t*)data;
+	if(nd->rcv==NULL || nd->data.s==NULL || nd->data.len<=0)
+		return -1;
+
+	memset(&sto, 0, sizeof(struct _siptrace_data));
+
+	sto.body.s   = nd->data.s;
+	sto.body.len = nd->data.len;
+
+	siptrace_copy_proto(nd->rcv->proto, sto.fromip_buff);
+	strcat(sto.fromip_buff, ip_addr2a(&nd->rcv->src_ip));
+	strcat(sto.fromip_buff,":");
+	strcat(sto.fromip_buff, int2str(nd->rcv->src_port, NULL));
+	sto.fromip.s = sto.fromip_buff;
+	sto.fromip.len = strlen(sto.fromip_buff);
+
+	siptrace_copy_proto(nd->rcv->proto, sto.toip_buff);
+	strcat(sto.toip_buff, ip_addr2a(&nd->rcv->dst_ip));
+	strcat(sto.toip_buff,":");
+	strcat(sto.toip_buff, int2str(nd->rcv->dst_port, NULL));
+	sto.toip.s = sto.toip_buff;
+	sto.toip.len = strlen(sto.toip_buff);
+
+	sto.dir = "in";
+
+	trace_send_hep_duplicate(&sto.body, &sto.fromip, &sto.toip, NULL);
+	return 0;
+
+}
+
+/**
+ *
+ */
+int siptrace_net_data_send(void *data)
+{
+	sr_net_info_t *nd;
+	struct dest_info new_dst;
+	struct _siptrace_data sto;
+
+	if(data==0)
+		return -1;
+
+	nd = (sr_net_info_t*)data;
+	if(nd->dst==NULL || nd->data.s==NULL || nd->data.len<=0)
+		return -1;
+
+	new_dst=*nd->dst;
+	new_dst.send_sock=get_send_socket(0, &nd->dst->to, nd->dst->proto);
+
+	memset(&sto, 0, sizeof(struct _siptrace_data));
+
+	sto.body.s   = nd->data.s;
+	sto.body.len = nd->data.len;
+
+	if (unlikely(new_dst.send_sock==0)) {
+		LM_WARN("no sending socket found\n");
+		strcpy(sto.fromip_buff, "any:255.255.255.255:5060");
+	} else {
+		strncpy(sto.fromip_buff, new_dst.send_sock->sock_str.s,
+			new_dst.send_sock->sock_str.len);
+	}
+	sto.fromip.s = sto.fromip_buff;
+	sto.fromip.len = strlen(sto.fromip_buff);
+
+	siptrace_copy_proto(new_dst.send_sock->proto, sto.toip_buff);
+	strcat(sto.toip_buff, suip2a(&new_dst.to, sizeof(new_dst.to)));
+	strcat(sto.toip_buff,":");
+	strcat(sto.toip_buff, int2str((int)su_getport(&new_dst.to), NULL));
+	sto.toip.s = sto.toip_buff;
+	sto.toip.len = strlen(sto.toip_buff);
+
+	sto.dir = "out";
+
+	trace_send_hep_duplicate(&sto.body, &sto.fromip, &sto.toip, NULL);
+	return 0;
+}
+
 static void siptrace_rpc_status (rpc_t* rpc, void* c) {
 	str status = {0, 0};
 
@@ -1911,8 +2049,8 @@ static void siptrace_rpc_status (rpc_t* rpc, void* c) {
 }
 
 static const char* siptrace_status_doc[2] = {
-        "Get status or turn on/off siptrace. Parameters: on, off or check.",
-        0
+	"Get status or turn on/off siptrace. Parameters: on, off or check.",
+	0
 };
 
 rpc_export_t siptrace_rpc[] = {
@@ -1929,4 +2067,3 @@ static int siptrace_init_rpc(void)
 	}
 	return 0;
 }
-
