@@ -1,17 +1,15 @@
 /*
- * $Id$
- *
  * Copyright (C) 2006 SOMA Networks, Inc.
  * Written by Ron Winacott (karwin)
  *
- * This file is part of SIP-router, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * SIP-router is free software; you can redistribute it and/or modify it
+ * Kamailio is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * SIP-router is distributed in the hope that it will be useful, but
+ * Kamailio is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
@@ -175,13 +173,19 @@ static inline int sst_build_minse_hdr(int seval, str *sehdr)
 	sehdr->s = sst_se_buf;
 	return 0;
 }
-static inline int sst_build_se_hdr(int seval, str *sehdr)
+static inline int sst_build_se_hdr(int seval, str *sehdr, char *refresher)
 {
 	if(sehdr==NULL)
 		return -1;
 
-	sehdr->len = snprintf(sst_se_buf, SST_SE_BUF_SIZE,
-			"Session-Expires: %d\r\n", seval);
+	if(refresher==NULL) {
+		sehdr->len = snprintf(sst_se_buf, SST_SE_BUF_SIZE,
+				"Session-Expires: %d\r\n", seval);
+	}
+	else {
+		sehdr->len = snprintf(sst_se_buf, SST_SE_BUF_SIZE,
+				"Session-Expires: %d;refresher=%s\r\n", seval, refresher);
+	}
 	sehdr->s = sst_se_buf;
 	return 0;
 }
@@ -344,7 +348,7 @@ void sst_dialog_created_CB(struct dlg_cell *did, int type,
 		}
 		
 		info->requester = SST_PXY;
-		sst_build_se_hdr(info->interval, &msehdr);
+		sst_build_se_hdr(info->interval, &msehdr, NULL);
 		if (append_header(msg, msehdr.s)) {
 			LM_ERR("failed to append Session-Expires header to proxy "
 					"requested SST.\n");
@@ -564,7 +568,7 @@ static void sst_dialog_response_fwded_CB(struct dlg_cell* did, int type,
 					 * header and forward back to the UAC and it will
 					 * deal with refreshing the session.
 					 */
-					sst_build_se_hdr(info->interval, &sehdr);
+					sst_build_se_hdr(info->interval, &sehdr, "uac");
 					if (append_header(msg, sehdr.s)) {
 						LM_ERR("failed to append Session-Expires header\n");
 						return;
@@ -671,7 +675,7 @@ int sst_check_min(struct sip_msg *msg, char *flag, char *str2)
 			 * Too small. See if we need to send the 422 and are able
 			 * to send it.
 			 */
-			if (flag) {
+			if (flag && *flag) {
 				str msehdr;
 				sst_build_minse_hdr(sst_min_se, &msehdr);
 				LM_DBG("Sending 422: %.*s\n", msehdr.len, msehdr.s);
