@@ -1,23 +1,16 @@
 /*
- * $Id$
- *
  * message printing
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -26,27 +19,6 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- * History:
- * ----------
- * 2003-01-27  next baby-step to removing ZT - PRESERVE_ZT (jiri)
- * 2003-02-13  build_uac_request uses proto (andrei)
- * 2003-02-28  scratchpad compatibility abandoned (jiri)
- * 2003-04-14  build_local no longer checks reply status as it
- *             is now called before reply status is updated to
- *             avoid late ACK sending (jiri)
- * 2003-10-02  added via_builder set host/port support (andrei)
- * 2004-02-11  FIFO/CANCEL + alignments (hash=f(callid,cseq)) (uli+jiri)
- * 2004-02-13: t->is_invite and t->local replaced with flags (bogdan)
- * 2006-04-21  build_uac_req, assemble_via use struct dest_info now;
- *              uri2sock replaced with uri2dst (andrei)
- * 2006-08-11  build_dlg_ack: use the first dns ip for which a send_sock
- *              is found (andrei)
- * 2007-03-15  build_dls_ack: removed next_hop and replaced by dst to avoid
- *               resolving nexthop twice (andrei)
- * 2007-05-28: build_local_reparse() is introdued: it uses the outgoing
- *             INVITE as a source to construct a CANCEL or ACK (Miklos)
- * 2010-02-26  cancel reason (rfc3326) basic support (andrei)
  */
 
 #include "defs.h"
@@ -65,6 +37,7 @@
 #include "../../ut.h"
 #include "../../parser/msg_parser.h"
 #include "../../parser/contact/parse_contact.h"
+#include "../../srapi.h"
 #include "lw_parser.h"
 #include "t_msgbuilder.h"
 #include "uac.h"
@@ -226,8 +199,8 @@ char *build_local(struct cell *Trans,unsigned int branch,
 	append_str( p, Trans->cseq_n.s, Trans->cseq_n.len );
 	append_str( p, " ", 1 );
 	append_str( p, method, method_len );
-	append_str( p, MAXFWD_HEADER, MAXFWD_HEADER_LEN );
 	append_str( p, CRLF, CRLF_LEN );
+	append_str( p, MAXFWD_HEADER, MAXFWD_HEADER_LEN );
 
 	if (!is_local(Trans))  {
 		for ( hdr=Trans->uas.request->headers ; hdr ; hdr=hdr->next )
@@ -307,6 +280,8 @@ char *build_local_reparse(struct cell *Trans,unsigned int branch,
 	int reason_len, code_len;
 	struct hdr_field *reas1, *reas_last, *hdr;
 #endif /* CANCEL_REASON_SUPPORT */
+	int hadded = 0;
+	sr_cfgenv_t *cenv = NULL;
 
 	invite_buf = Trans->uac[branch].request.buffer;
 	invite_len = Trans->uac[branch].request.buffer_len;
@@ -384,6 +359,8 @@ char *build_local_reparse(struct cell *Trans,unsigned int branch,
 	s = eat_line(s, invite_buf_end - s);
 	append_str(d, s1, s - s1);
 
+	cenv = sr_cfgenv_get();
+
 	/* check every header field name,
 	we must exclude and modify some of the headers */
 	first_via = 1;
@@ -501,15 +478,35 @@ char *build_local_reparse(struct cell *Trans,unsigned int branch,
 
 			default:
 				s = lw_next_line(s, invite_buf_end);
+				hadded = 0;
 
-				if (cfg_get(tm, tm_cfg, ac_extra_hdrs).len
-				&& (s1 + cfg_get(tm, tm_cfg, ac_extra_hdrs).len < invite_buf_end)
-				&& (strncasecmp(s1,
-						cfg_get(tm, tm_cfg, ac_extra_hdrs).s,
-						cfg_get(tm, tm_cfg, ac_extra_hdrs).len) == 0)
-				) {
-					append_str(d, s1, s - s1);
-				} /* else skip this line */
+				/* uac auth headers */
+				if(Trans->uas.request &&
+						(Trans->uas.request->msg_flags & FL_UAC_AUTH)) {
+					if(s1 + cenv->uac_cseq_auth.len + 2 < invite_buf_end) {
+						if(s1[cenv->uac_cseq_auth.len]==':'
+								&& strncmp(s1, cenv->uac_cseq_auth.s,
+									cenv->uac_cseq_auth.len)==0) {
+							hadded = 1;
+							append_str(d, s1, s - s1);
+						} else if(s1[cenv->uac_cseq_refresh.len]==':'
+								&& strncmp(s1, cenv->uac_cseq_refresh.s,
+									cenv->uac_cseq_refresh.len)==0) {
+							hadded = 1;
+							append_str(d, s1, s - s1);
+						}
+					}
+				}
+
+				if(likely(hadded==0)) {
+					if (cfg_get(tm, tm_cfg, ac_extra_hdrs).len
+							&& (s1 + cfg_get(tm, tm_cfg, ac_extra_hdrs).len < invite_buf_end)
+							&& (strncasecmp(s1,
+									cfg_get(tm, tm_cfg, ac_extra_hdrs).s,
+									cfg_get(tm, tm_cfg, ac_extra_hdrs).len) == 0)) {
+						append_str(d, s1, s - s1);
+					}
+				}
 				break;
 		}
 	}
@@ -1484,29 +1481,6 @@ static inline char* print_callid(char* w, dlg_t* dialog, struct cell* t)
 	return w;
 }
 
-/*
-* Find the first occurrence of find in s, where the search is limited to the
-* first slen characters of s.
-*/
-static
-char * _strnstr(const char* s, const char* find, size_t slen) {
-	char c, sc;
-	size_t len;
-
-	if ((c = *find++) != '\0') {
-		len = strlen(find);
-		do {
-			do {
-				if (slen-- < 1 || (sc = *s++) == '\0')
-					return (NULL);
-			} while (sc != c);
-			if (len > slen)
-				return (NULL);
-		} while (strncmp(s, find, len) != 0);
-		s--;
-	}
-	return ((char *)s);
-}
 
 /*
  * Create a request
@@ -1518,11 +1492,20 @@ char* build_uac_req(str* method, str* headers, str* body, dlg_t* dialog, int bra
 	str content_length, cseq, via;
 	unsigned int maxfwd_len;
 	int tbracket, fbracket;
+	str fromtag = STR_NULL;
+	str loc_tag = STR_NULL;
 
 	if (!method || !dialog) {
 		LOG(L_ERR, "build_uac_req(): Invalid parameter value\n");
 		return 0;
 	}
+
+	if (dialog->id.loc_tag.len<=0) {
+		/* From Tag is mandatory in RFC3261 - generate one if not provided */
+		generate_fromtag(&fromtag, &dialog->id.call_id);
+		loc_tag = dialog->id.loc_tag;
+		dialog->id.loc_tag = fromtag;
+	}
 	if (print_content_length(&content_length, body) < 0) {
 		LOG(L_ERR, "build_uac_req(): Error while printing content-length\n");
 		return 0;
@@ -1603,6 +1586,9 @@ char* build_uac_req(str* method, str* headers, str* body, dlg_t* dialog, int bra
 	memapp(w, via.s, via.len);                            /* Top-most Via */
 	w = print_to(w, dialog, t, tbracket);                 /* To */
 	w = print_from(w, dialog, t, fbracket);               /* From */
+	if(fromtag.len>0) {
+		dialog->id.loc_tag = loc_tag;
+	}
 	w = print_cseq(w, &cseq, method, t);                  /* CSeq */
 	w = print_callid(w, dialog, t);                       /* Call-ID */
 	w = print_routeset(w, dialog);                        /* Route set */
