@@ -1,9 +1,8 @@
 /**
- * $Id$
  *
  * Copyright (C) 2009 SIP-Router.org
  *
- * This file is part of Extensible SIP Router, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -391,6 +390,14 @@ int th_unmask_via(sip_msg_t *msg, str *cookie)
 			LM_DBG("body: %d: [%.*s]\n", vlen, vlen, via->name.s);
 			if(i!=1)
 			{
+				/* Skip if via is not encoded */
+				if (via->host.len!=th_ip.len
+						|| strncasecmp(via->host.s, th_ip.s, th_ip.len)!=0)
+				{
+					LM_DBG("via %d is not encoded",i);
+					continue;
+				}
+
 				vp = th_get_via_param(via, &th_vparam_name);
 				if(vp==NULL)
 				{
@@ -405,7 +412,7 @@ int th_unmask_via(sip_msg_t *msg, str *cookie)
 							&th_vparam_prefix, 0, &out.len);
 				if(out.s==NULL)
 				{
-					LM_ERR("cannot encode via %d\n", i);
+					LM_ERR("cannot decode via %d\n", i);
 					return -1;
 				}
 					
@@ -476,6 +483,14 @@ int th_unmask_callid(sip_msg_t *msg)
 		LM_ERR("cannot get Call-Id header\n");
 		return -1;
 	}
+
+	/* Do nothing if call-id is not encoded */
+	if ((msg->callid->body.len<th_callid_prefix.len) ||
+			(strncasecmp(msg->callid->body.s,th_callid_prefix.s,th_callid_prefix.len)!=0))
+	{
+		LM_DBG("call-id [%.*s] not encoded",msg->callid->body.len,msg->callid->body.s);
+		return 0;
+	}
 				
 	out.s = th_mask_decode(msg->callid->body.s, msg->callid->body.len,
 					&th_callid_prefix, 0, &out.len);
@@ -501,6 +516,47 @@ int th_unmask_callid(sip_msg_t *msg)
 	return 0;
 }
 
+#define TH_CALLID_SIZE	256
+int th_unmask_callid_str(str *icallid, str *ocallid)
+{
+	static char th_callid_buf[TH_CALLID_SIZE];
+	str out;
+
+	if(th_param_mask_callid==0)
+		return 0;
+
+	if(icallid->s==NULL) {
+		LM_ERR("invalid Call-Id value\n");
+		return -1;
+	}
+
+	if(th_callid_prefix.len>0) {
+		if(th_callid_prefix.len >= icallid->len) {
+			return 1;
+		}
+		if(strncmp(icallid->s, th_callid_prefix.s, th_callid_prefix.len)!=0) {
+			return 1;
+		}
+	}
+	out.s = th_mask_decode(icallid->s, icallid->len,
+					&th_callid_prefix, 0, &out.len);
+	if(out.len>=TH_CALLID_SIZE) {
+		pkg_free(out.s);
+		LM_ERR("not enough callid buf size (needed %d)\n", out.len);
+		return -2;
+	}
+
+	memcpy(th_callid_buf, out.s, out.len);
+	th_callid_buf[out.len] = '\0';
+
+	pkg_free(out.s);
+
+	ocallid->s = th_callid_buf;
+	ocallid->len = out.len;
+
+	return 0;
+}
+
 int th_flip_record_route(sip_msg_t *msg, int mode)
 {
 	hdr_field_t *hdr;
@@ -630,6 +686,15 @@ int th_unmask_route(sip_msg_t *msg)
 			i++;
 			if(i!=1)
 			{
+				/* Skip if route is not encoded */
+				if ((rr->nameaddr.uri.len<th_uri_prefix.len) ||
+						(strncasecmp(rr->nameaddr.uri.s,th_uri_prefix.s,th_uri_prefix.len)!=0))
+				{
+					LM_DBG("rr %d is not encoded: [%.*s]",i,rr->nameaddr.uri.len,rr->nameaddr.uri.s);
+					rr = rr->next;
+					continue;
+				}
+
 				if(th_get_uri_param_value(&rr->nameaddr.uri, &th_uparam_name,
 							&eval)<0 || eval.len<=0)
 					return -1;
@@ -670,6 +735,14 @@ int th_unmask_ruri(sip_msg_t *msg)
 	struct lump* l;
 	str out;
 
+	/* Do nothing if ruri is not encoded */
+	if ((REQ_LINE(msg).uri.len<th_uri_prefix.len) ||
+			(strncasecmp(REQ_LINE(msg).uri.s,th_uri_prefix.s,th_uri_prefix.len)!=0))
+	{
+		LM_DBG("ruri [%.*s] is not encoded",REQ_LINE(msg).uri.len,REQ_LINE(msg).uri.s);
+		return 0;
+	}
+
 	if(th_get_uri_param_value(&REQ_LINE(msg).uri, &th_uparam_name, &eval)<0
 			|| eval.len<=0)
 		return -1;
@@ -723,6 +796,15 @@ int th_unmask_refer_to(sip_msg_t *msg)
 	}
 
 	uri = &(get_refer_to(msg)->uri);
+
+	/* Do nothing if refer_to is not encoded */
+	if ((uri->len<th_uri_prefix.len)
+			|| (strncasecmp(uri->s, th_uri_prefix.s, th_uri_prefix.len)!=0))
+	{
+		LM_DBG("refer-to [%.*s] is not encoded",uri->len,uri->s);
+		return 0;
+	}
+
 	if(th_get_uri_param_value(uri, &th_uparam_name, &eval)<0
 			|| eval.len<=0)
 		return -1;
@@ -914,6 +996,10 @@ int th_add_hdr_cookie(sip_msg_t *msg)
 struct via_param *th_get_via_cookie(sip_msg_t *msg, struct via_body *via)
 {
 	struct via_param *p;
+
+	if (!via) {
+		return NULL;
+	}
 	for(p=via->param_lst; p; p=p->next)
 	{
 		if(p->name.len==th_cookie_name.len
@@ -1002,6 +1088,15 @@ int th_del_cookie(sip_msg_t *msg)
 }
 
 
+/**
+ * return the special topoh cookie
+ * - TH header of TH Via parame
+ * - value is 3 chars
+ *   [0] - direction:    d - downstream; u - upstream
+ *   [1] - request type: i - initial; c - in-dialog; l - local in-dialog
+ *   [2] - location:     h - header; v - via param
+ * - if not found, returns 'xxx'
+ */
 char* th_get_cookie(sip_msg_t *msg, int *clen)
 {
 	hdr_field_t *hf;
