@@ -1,44 +1,24 @@
 /*
- * $Id$
- *
  * sip msg. header proxy parser
  *
- *
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * History:
- * ---------
- *  2003-02-28  scratchpad compatibility abandoned (jiri)
- *  2003-01-29  scrathcpad removed (jiri)
- *  2003-01-27  next baby-step to removing ZT - PRESERVE_ZT (jiri)
- *  2003-03-31  removed msg->repl_add_rm (andrei)
- *  2003-04-26 ZSW (jiri)
- *  2003-05-01  parser extended to support Accept header field (janakj)
- *  2005-02-23  parse_headers uses hdr_flags_t now (andrei)
- *  2005-03-02  free_via_list(vb) on via parse error (andrei)
- *  2007-01-26  parser extended to support Identity, Identity-info and Date
- *		header fields (gergo)
  */
 
 /** Parser :: SIP Message header proxy parser.
@@ -47,7 +27,7 @@
  */
 
 /*! \defgroup parser SIP-router SIP message parser
- * 
+ *
  * The SIP message parser
  *
  */
@@ -88,7 +68,7 @@ unsigned int global_req_flags = 0;
 
 /* returns pointer to next header line, and fill hdr_f ;
  * if at end of header returns pointer to the last crlf  (always buf)*/
-char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
+char* get_hdr_field(char* const buf, char* const end, struct hdr_field* const hdr)
 {
 
 	char *tmp = 0;
@@ -99,6 +79,11 @@ char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
 	int integer, err;
 	unsigned uval;
 
+	if(!buf) {
+		DBG("null buffer pointer\n");
+		goto error;
+	}
+
 	if ((*buf)=='\n' || (*buf)=='\r'){
 		/* double crlf or lflf or crcr */
 		DBG("found end of header\n");
@@ -108,14 +93,14 @@ char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
 
 	tmp=parse_hname(buf, end, hdr);
 	if (hdr->type==HDR_ERROR_T){
-		LOG(L_ERR, "ERROR: get_hdr_field: bad header\n");
+		ERR("bad header\n");
 		goto error;
 	}
 
 	/* eliminate leading whitespace */
 	tmp=eat_lws_end(tmp, end);
 	if (tmp>=end) {
-		LOG(L_ERR, "ERROR: get_hdr_field: HF empty\n");
+		ERR("HF empty\n");
 		goto error;
 	}
 
@@ -130,14 +115,14 @@ char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
 			via_cnt++;
 			vb=pkg_malloc(sizeof(struct via_body));
 			if (vb==0){
-				LOG(L_ERR, "get_hdr_field: out of memory\n");
+				ERR("out of memory\n");
 				goto error;
 			}
 			memset(vb,0,sizeof(struct via_body));
 			hdr->body.s=tmp;
 			tmp=parse_via(tmp, end, vb);
 			if (vb->error==PARSE_ERROR){
-				LOG(L_ERR, "ERROR: get_hdr_field: bad via\n");
+				ERR("bad via\n");
 				free_via_list(vb);
 				goto error;
 			}
@@ -149,69 +134,64 @@ char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
 		case HDR_CSEQ_T:
 			cseq_b=pkg_malloc(sizeof(struct cseq_body));
 			if (cseq_b==0){
-				LOG(L_ERR, "get_hdr_field: out of memory\n");
+				ERR("out of memory\n");
 				goto error;
 			}
 			memset(cseq_b, 0, sizeof(struct cseq_body));
 			hdr->body.s=tmp;
 			tmp=parse_cseq(tmp, end, cseq_b);
 			if (cseq_b->error==PARSE_ERROR){
-				LOG(L_ERR, "ERROR: get_hdr_field: bad cseq\n");
+				ERR("bad cseq\n");
 				free_cseq(cseq_b);
 				goto error;
 			}
 			hdr->parsed=cseq_b;
 			hdr->body.len=tmp-hdr->body.s;
-			DBG("get_hdr_field: cseq <%.*s>: <%.*s> <%.*s>\n",
-					hdr->name.len, ZSW(hdr->name.s),
-					cseq_b->number.len, ZSW(cseq_b->number.s),
-					cseq_b->method.len, cseq_b->method.s);
+			DBG("cseq <%.*s>: <%.*s> <%.*s>\n", hdr->name.len, ZSW(hdr->name.s),
+					cseq_b->number.len, ZSW(cseq_b->number.s), cseq_b->method.len,
+					cseq_b->method.s);
 			break;
 		case HDR_TO_T:
 			to_b=pkg_malloc(sizeof(struct to_body));
 			if (to_b==0){
-				LOG(L_ERR, "get_hdr_field: out of memory\n");
+				ERR("out of memory\n");
 				goto error;
 			}
 			memset(to_b, 0, sizeof(struct to_body));
 			hdr->body.s=tmp;
 			tmp=parse_to(tmp, end,to_b);
 			if (to_b->error==PARSE_ERROR){
-				LOG(L_ERR, "ERROR: get_hdr_field: bad to header\n");
+				ERR("bad to header\n");
 				free_to(to_b);
 				goto error;
 			}
 			hdr->parsed=to_b;
 			hdr->body.len=tmp-hdr->body.s;
-			DBG("DEBUG: get_hdr_field: <%.*s> [%d]; uri=[%.*s] \n",
-				hdr->name.len, ZSW(hdr->name.s),
-				hdr->body.len, to_b->uri.len,ZSW(to_b->uri.s));
-			DBG("DEBUG: to body [%.*s]\n",to_b->body.len,
-				ZSW(to_b->body.s));
+			DBG("<%.*s> [%d]; uri=[%.*s]\n", hdr->name.len, ZSW(hdr->name.s),
+					hdr->body.len, to_b->uri.len, ZSW(to_b->uri.s));
+			DBG("to body [%.*s]\n", to_b->body.len, ZSW(to_b->body.s));
 			break;
 		case HDR_CONTENTLENGTH_T:
 			hdr->body.s=tmp;
 			tmp=parse_content_length(tmp,end, &integer);
 			if (tmp==0){
-				LOG(L_ERR, "ERROR:get_hdr_field: bad content_length header\n");
+				ERR("bad content_length header\n");
 				goto error;
 			}
 			hdr->parsed=(void*)(long)integer;
 			hdr->body.len=tmp-hdr->body.s;
-			DBG("DEBUG: get_hdr_body : content_length=%d\n",
-					(int)(long)hdr->parsed);
+			DBG("content_length=%d\n", (int)(long)hdr->parsed);
 			break;
 		case HDR_RETRY_AFTER_T:
 			hdr->body.s=tmp;
 			tmp=parse_retry_after(tmp,end, &uval, &err);
 			if (err){
-				LOG(L_ERR, "ERROR:get_hdr_field: bad retry_after header\n");
+				ERR("bad retry_after header\n");
 				goto error;
 			}
 			hdr->parsed=(void*)(unsigned long)uval;
 			hdr->body.len=tmp-hdr->body.s;
-			DBG("DEBUG: get_hdr_body : retry_after=%d\n",
-					(unsigned)(long)hdr->parsed);
+			DBG("retry_after=%d\n", (unsigned)(long)hdr->parsed);
 			break;
 		case HDR_IDENTITY_T:
 		case HDR_DATE_T:
@@ -270,9 +250,7 @@ char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
 				if (match){
 					match++;
 				}else {
-					LOG(L_ERR,
-							"ERROR: get_hdr_field: bad body for <%s>(%d)\n",
-							hdr->name.s, hdr->type);
+					ERR("bad body for <%s>(%d)\n", hdr->name.s, hdr->type);
 					/* abort(); */
 					tmp=end;
 					goto error;
@@ -283,8 +261,7 @@ char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
 			hdr->body.len=match-hdr->body.s;
 			break;
 		default:
-			LOG(L_CRIT, "BUG: get_hdr_field: unknown header type %d\n",
-					hdr->type);
+			BUG("unknown header type %d\n", hdr->type);
 			goto error;
 	}
 	/* jku: if \r covered by current length, shrink it */
@@ -292,7 +269,7 @@ char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
 	hdr->len=tmp-hdr->name.s;
 	return tmp;
 error:
-	DBG("get_hdr_field: error exit\n");
+	DBG("error exit\n");
 	STATS_BAD_MSG_HDR();
 	hdr->type=HDR_ERROR_T;
 	hdr->len=tmp-hdr->name.s;
@@ -316,7 +293,7 @@ error:
    give you the first occurrence of a header you are interested in,
    look at check_transaction_quadruple
 */
-int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
+int parse_headers(struct sip_msg* const msg, const hdr_flags_t flags, const int next)
 {
 	struct hdr_field* hf;
 	char* tmp;
@@ -334,14 +311,14 @@ int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
 		orig_flag=0;
 
 #ifdef EXTRA_DEBUG
-	DBG("parse_headers: flags=%llx\n", (unsigned long long)flags);
+	DBG("flags=%llx\n", (unsigned long long)flags);
 #endif
 	while( tmp<end && (flags & msg->parsed_flag) != flags){
 		prefetch_loc_r(tmp+64, 1);
 		hf=pkg_malloc(sizeof(struct hdr_field));
 		if (unlikely(hf==0)){
 			ser_error=E_OUT_OF_MEM;
-			LOG(L_ERR, "ERROR:parse_headers: memory allocation error\n");
+			ERR("memory allocation error\n");
 			goto error;
 		}
 		memset(hf,0, sizeof(struct hdr_field));
@@ -349,8 +326,7 @@ int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
 		rest=get_hdr_field(tmp, end, hf);
 		switch (hf->type){
 			case HDR_ERROR_T:
-				LOG(L_INFO,"ERROR: bad header field [%.*s]\n",
-					(end-tmp>20)?20:(int)(end-tmp), tmp);
+				ERR("bad header field [%.*s]\n", (end-tmp>20)?20:(int)(end-tmp), tmp);
 				goto  error;
 			case HDR_EOH_T:
 				msg->eoh=tmp; /* or rest?*/
@@ -507,10 +483,9 @@ int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
 				break;
 			case HDR_VIA_T:
 				msg->parsed_flag|=HDR_VIA_F;
-				DBG("parse_headers: Via found, flags=%llx\n",
-						(unsigned long long)flags);
+				DBG("Via found, flags=%llx\n", (unsigned long long)flags);
 				if (msg->via1==0) {
-					DBG("parse_headers: this is the first via\n");
+					DBG("this is the first via\n");
 					msg->h_via1=hf;
 					msg->via1=hf->parsed;
 					if (msg->via1->next){
@@ -521,7 +496,7 @@ int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
 					msg->h_via2=hf;
 					msg->via2=hf->parsed;
 					msg->parsed_flag|=HDR_VIA2_F;
-					DBG("parse_headers: this is the second via\n");
+					DBG("this is the second via\n");
 				}
 				break;
 			case HDR_DATE_T:
@@ -556,8 +531,7 @@ int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
 				msg->parsed_flag|=HDR_REASON_F;
 				break;
 			default:
-				LOG(L_CRIT, "BUG: parse_headers: unknown header type %d\n",
-							hf->type);
+				BUG("unknown header type %d\n", hf->type);
 				goto error;
 		}
 		/* add the header to the list*/
@@ -569,10 +543,8 @@ int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
 			msg->last_header=hf;
 		}
 #ifdef EXTRA_DEBUG
-		DBG("header field type %d, name=<%.*s>, body=<%.*s>\n",
-			hf->type,
-			hf->name.len, ZSW(hf->name.s),
-			hf->body.len, ZSW(hf->body.s));
+		DBG("header field type %d, name=<%.*s>, body=<%.*s>\n", hf->type,
+				hf->name.len, ZSW(hf->name.s), hf->body.len, ZSW(hf->body.s));
 #endif
 		tmp=rest;
 	}
@@ -595,7 +567,7 @@ error:
 
 
 /* returns 0 if ok, -1 for errors */
-int parse_msg(char* buf, unsigned int len, struct sip_msg* msg)
+int parse_msg(char* const buf, const unsigned int len, struct sip_msg* const msg)
 {
 
 	char *tmp;
@@ -604,35 +576,34 @@ int parse_msg(char* buf, unsigned int len, struct sip_msg* msg)
 	int offset;
 	hdr_flags_t flags;
 
-	/* eat crlf from the beginning */
-	for (tmp=buf; (*tmp=='\n' || *tmp=='\r')&&
-			tmp-buf < len ; tmp++);
+	/* eat crlf & whitespaces from the beginning */
+	for (tmp=buf; (tmp-buf < len)
+			&& (*tmp=='\n' || *tmp=='\r' || *tmp=='\0'
+				|| *tmp=='\t' || *tmp==' ');
+			tmp++);
 	offset=tmp-buf;
 	fl=&(msg->first_line);
 	rest=parse_first_line(tmp, len-offset, fl);
-#if 0
-	rest=parse_fline(tmp, buf+len, fl);
-#endif
 	offset+=rest-tmp;
 	tmp=rest;
 	switch(fl->type){
 		case SIP_INVALID:
-			DBG("parse_msg: invalid message\n");
+			DBG("invalid message\n");
 			goto error;
 			break;
 		case SIP_REQUEST:
 			DBG("SIP Request:\n");
-			DBG(" method:  <%.*s>\n",fl->u.request.method.len,
-				ZSW(fl->u.request.method.s));
-			DBG(" uri:     <%.*s>\n",fl->u.request.uri.len,
-				ZSW(fl->u.request.uri.s));
-			DBG(" version: <%.*s>\n",fl->u.request.version.len,
-				ZSW(fl->u.request.version.s));
+			DBG(" method:  <%.*s>\n", fl->u.request.method.len,
+					ZSW(fl->u.request.method.s));
+			DBG(" uri:     <%.*s>\n", fl->u.request.uri.len,
+					ZSW(fl->u.request.uri.s));
+			DBG(" version: <%.*s>\n", fl->u.request.version.len,
+					ZSW(fl->u.request.version.s));
 			flags=HDR_VIA_F;
 			break;
 		case SIP_REPLY:
 			DBG("SIP Reply  (status):\n");
-			DBG(" version: <%.*s>\n",fl->u.reply.version.len,
+			DBG(" version: <%.*s>\n", fl->u.reply.version.len,
 					ZSW(fl->u.reply.version.s));
 			DBG(" status:  <%.*s>\n", fl->u.reply.status.len,
 					ZSW(fl->u.reply.status.s));
@@ -643,7 +614,7 @@ int parse_msg(char* buf, unsigned int len, struct sip_msg* msg)
 			flags=HDR_VIA_F;
 			break;
 		default:
-			DBG("unknown type %d\n",fl->type);
+			DBG("unknown type %d\n", fl->type);
 			goto error;
 	}
 	msg->unparsed=tmp;
@@ -654,41 +625,30 @@ int parse_msg(char* buf, unsigned int len, struct sip_msg* msg)
 	/* dump parsed data */
 	if (msg->via1){
 		DBG("first via: <%.*s/%.*s/%.*s> <%.*s:%.*s(%d)>",
-			msg->via1->name.len,
-			ZSW(msg->via1->name.s),
-			msg->via1->version.len,
-			ZSW(msg->via1->version.s),
-			msg->via1->transport.len,
-			ZSW(msg->via1->transport.s),
-			msg->via1->host.len,
-			ZSW(msg->via1->host.s),
-			msg->via1->port_str.len,
-			ZSW(msg->via1->port_str.s),
-			msg->via1->port);
-		if (msg->via1->params.s)  DBG(";<%.*s>",
-				msg->via1->params.len, ZSW(msg->via1->params.s));
+				msg->via1->name.len, ZSW(msg->via1->name.s),
+				msg->via1->version.len, ZSW(msg->via1->version.s),
+				msg->via1->transport.len, ZSW(msg->via1->transport.s),
+				msg->via1->host.len, ZSW(msg->via1->host.s),
+				msg->via1->port_str.len, ZSW(msg->via1->port_str.s),
+				msg->via1->port);
+		if (msg->via1->params.s)
+			DBG(";<%.*s>", msg->via1->params.len, ZSW(msg->via1->params.s));
 		if (msg->via1->comment.s)
-				DBG(" <%.*s>",
-					msg->via1->comment.len, ZSW(msg->via1->comment.s));
+			DBG(" <%.*s>", msg->via1->comment.len, ZSW(msg->via1->comment.s));
 		DBG ("\n");
 	}
 	if (msg->via2){
 		DBG("second via: <%.*s/%.*s/%.*s> <%.*s:%.*s(%d)>",
-			msg->via2->name.len,
-			ZSW(msg->via2->name.s),
-			msg->via2->version.len,
-			ZSW(msg->via2->version.s),
-			msg->via2->transport.len,
-			ZSW(msg->via2->transport.s),
-			msg->via2->host.len,
-			ZSW(msg->via2->host.s),
-			msg->via2->port_str.len,
-			ZSW(msg->via2->port_str.s),
-			msg->via2->port);
-		if (msg->via2->params.s)  DBG(";<%.*s>",
-				msg->via2->params.len, ZSW(msg->via2->params.s));
-		if (msg->via2->comment.s) DBG(" <%.*s>",
-				msg->via2->comment.len, ZSW(msg->via2->comment.s));
+				msg->via2->name.len, ZSW(msg->via2->name.s),
+				msg->via2->version.len, ZSW(msg->via2->version.s),
+				msg->via2->transport.len, ZSW(msg->via2->transport.s),
+				msg->via2->host.len, ZSW(msg->via2->host.s),
+				msg->via2->port_str.len, ZSW(msg->via2->port_str.s),
+				msg->via2->port);
+		if (msg->via2->params.s)
+			DBG(";<%.*s>", msg->via2->params.len, ZSW(msg->via2->params.s));
+		if (msg->via2->comment.s)
+			DBG(" <%.*s>", msg->via2->comment.len, ZSW(msg->via2->comment.s));
 		DBG ("\n");
 	}
 #endif
@@ -722,32 +682,49 @@ void free_reply_lump( struct lump_rpl *lump)
 
 
 /*only the content*/
-void free_sip_msg(struct sip_msg* msg)
+void free_sip_msg(struct sip_msg* const msg)
 {
-	if (msg->new_uri.s) { pkg_free(msg->new_uri.s); msg->new_uri.len=0; }
-	if (msg->dst_uri.s) { pkg_free(msg->dst_uri.s); msg->dst_uri.len=0; }
-	if (msg->path_vec.s) { pkg_free(msg->path_vec.s); msg->path_vec.len=0; }
+	reset_new_uri(msg);
+	reset_dst_uri(msg);
+	reset_path_vector(msg);
+	reset_instance(msg);
+	reset_ruid(msg);
+	reset_ua(msg);
 	if (msg->headers)     free_hdr_field_lst(msg->headers);
 	if (msg->body && msg->body->free) msg->body->free(&msg->body);
 	if (msg->add_rm)      free_lump_list(msg->add_rm);
 	if (msg->body_lumps)  free_lump_list(msg->body_lumps);
 	if (msg->reply_lump)   free_reply_lump(msg->reply_lump);
+	msg_ldata_reset(msg);
 	/* don't free anymore -- now a pointer to a static buffer */
 #	ifdef DYN_BUF
 	pkg_free(msg->buf);
 #	endif
 }
 
+/**
+ * reset new uri value
+ */
+void reset_new_uri(struct sip_msg* const msg)
+{
+	if(msg->new_uri.s != 0) {
+		pkg_free(msg->new_uri.s);
+	}
+	msg->new_uri.s = 0;
+	msg->new_uri.len = 0;
+	msg->parsed_uri_ok = 0;
+}
+
 
 /*
  * Make a private copy of the string and assign it to dst_uri
  */
-int set_dst_uri(struct sip_msg* msg, str* uri)
+int set_dst_uri(struct sip_msg* const msg, const str* const uri)
 {
 	char* ptr;
 
 	if (unlikely(!msg || !uri)) {
-		LOG(L_ERR, "set_dst_uri: Invalid parameter value\n");
+		ERR("Invalid parameter value\n");
 		return -1;
 	}
 
@@ -759,7 +736,7 @@ int set_dst_uri(struct sip_msg* msg, str* uri)
 	} else {
 		ptr = (char*)pkg_malloc(uri->len);
 		if (!ptr) {
-			LOG(L_ERR, "set_dst_uri: Not enough memory\n");
+			ERR("Not enough memory\n");
 			return -1;
 		}
 
@@ -772,7 +749,7 @@ int set_dst_uri(struct sip_msg* msg, str* uri)
 }
 
 
-void reset_dst_uri(struct sip_msg* msg)
+void reset_dst_uri(struct sip_msg* const msg)
 {
 	if(msg->dst_uri.s != 0) {
 		pkg_free(msg->dst_uri.s);
@@ -786,7 +763,7 @@ int set_path_vector(struct sip_msg* msg, str* path)
 	char* ptr;
 
 	if (unlikely(!msg || !path)) {
-		LM_ERR("invalid parameter value\n");
+		ERR("invalid parameter value\n");
 		return -1;
 	}
 
@@ -798,7 +775,7 @@ int set_path_vector(struct sip_msg* msg, str* path)
 	} else {
 		ptr = (char*)pkg_malloc(path->len);
 		if (!ptr) {
-			LM_ERR("not enough pkg memory\n");
+			ERR("not enough pkg memory\n");
 			return -1;
 		}
 
@@ -811,17 +788,146 @@ int set_path_vector(struct sip_msg* msg, str* path)
 }
 
 
-void reset_path_vector(struct sip_msg* msg)
+void reset_path_vector(struct sip_msg* const msg)
 {
-	if(msg->path_vec.s != 0) {
-		pkg_free(msg->path_vec.s);
+	/* only free path vector from pkg IFF it is still in pkg... - ie. if msg is shm we don't free... */
+	if (!(msg->msg_flags&FL_SHM_CLONE)) {
+		if (msg->path_vec.s)
+			pkg_free(msg->path_vec.s);
+		msg->path_vec.s = 0;
+		msg->path_vec.len = 0;
 	}
-	msg->path_vec.s = 0;
-	msg->path_vec.len = 0;
 }
 
 
-hdr_field_t* get_hdr(sip_msg_t *msg, enum _hdr_types_t ht)
+int set_instance(struct sip_msg* msg, str* instance)
+{
+	char* ptr;
+
+	if (unlikely(!msg || !instance)) {
+		ERR("invalid instance parameter value\n");
+		return -1;
+	}
+
+	if (unlikely(instance->len == 0)) {
+		reset_instance(msg);
+	} else if (msg->instance.s && (msg->instance.len >= instance->len)) {
+		memcpy(msg->instance.s, instance->s, instance->len);
+		msg->instance.len = instance->len;
+	} else {
+		ptr = (char*)pkg_malloc(instance->len);
+		if (!ptr) {
+			ERR("not enough pkg memory for instance\n");
+			return -1;
+		}
+		memcpy(ptr, instance->s, instance->len);
+		if (msg->instance.s) pkg_free(msg->instance.s);
+		msg->instance.s = ptr;
+		msg->instance.len = instance->len;
+	}
+	return 0;
+}
+
+
+void reset_instance(struct sip_msg* const msg)
+{
+	if(msg->instance.s != 0) {
+		pkg_free(msg->instance.s);
+	}
+	msg->instance.s = 0;
+	msg->instance.len = 0;
+}
+
+
+int set_ruid(struct sip_msg* msg, str* ruid)
+{
+	char* ptr;
+
+	if (unlikely(!msg || !ruid)) {
+		ERR("invalid ruid parameter value\n");
+		return -1;
+	}
+
+	if (unlikely(ruid->len == 0)) {
+		reset_ruid(msg);
+	} else if (msg->ruid.s && (msg->ruid.len >= ruid->len)) {
+		memcpy(msg->ruid.s, ruid->s, ruid->len);
+		msg->ruid.len = ruid->len;
+	} else {
+		ptr = (char*)pkg_malloc(ruid->len);
+		if (!ptr) {
+			ERR("not enough pkg memory for ruid\n");
+			return -1;
+		}
+		memcpy(ptr, ruid->s, ruid->len);
+		if (msg->ruid.s) pkg_free(msg->ruid.s);
+		msg->ruid.s = ptr;
+		msg->ruid.len = ruid->len;
+	}
+	return 0;
+}
+
+
+void reset_ruid(struct sip_msg* const msg)
+{
+	if(msg->ruid.s != 0) {
+		pkg_free(msg->ruid.s);
+	}
+	msg->ruid.s = 0;
+	msg->ruid.len = 0;
+}
+
+
+int set_ua(struct sip_msg* msg, str* location_ua)
+{
+	char* ptr;
+
+	if (unlikely(!msg || !location_ua)) {
+		ERR("invalid location_ua parameter value\n");
+		return -1;
+	}
+
+	if (unlikely(location_ua->len == 0)) {
+		reset_ua(msg);
+	} else if (msg->location_ua.s && (msg->location_ua.len >= location_ua->len)) {
+		memcpy(msg->location_ua.s, location_ua->s, location_ua->len);
+		msg->location_ua.len = location_ua->len;
+	} else {
+		ptr = (char*)pkg_malloc(location_ua->len);
+		if (!ptr) {
+			ERR("not enough pkg memory for location_ua\n");
+			return -1;
+		}
+		memcpy(ptr, location_ua->s, location_ua->len);
+		if (msg->location_ua.s) pkg_free(msg->location_ua.s);
+		msg->location_ua.s = ptr;
+		msg->location_ua.len = location_ua->len;
+	}
+	return 0;
+}
+
+
+void reset_ua(struct sip_msg* const msg)
+{
+	if(msg->location_ua.s != 0) {
+		pkg_free(msg->location_ua.s);
+	}
+	msg->location_ua.s = 0;
+	msg->location_ua.len = 0;
+}
+
+/**
+ * reset content of msg->ldv (msg_ldata_t structure)
+ */
+void msg_ldata_reset(sip_msg_t *msg)
+{
+	if(msg==NULL)
+		return;
+	memset(&msg->ldv, 0, sizeof(msg_ldata_t));
+}
+
+
+hdr_field_t* get_hdr(const sip_msg_t* const msg, const enum _hdr_types_t ht)
 {
 	hdr_field_t *hdr;
 
@@ -833,7 +939,7 @@ hdr_field_t* get_hdr(sip_msg_t *msg, enum _hdr_types_t ht)
 }
 
 
-hdr_field_t* next_sibling_hdr(hdr_field_t *hf)
+hdr_field_t* next_sibling_hdr(const hdr_field_t* const hf)
 {
 	hdr_field_t *hdr;
 
@@ -843,7 +949,7 @@ hdr_field_t* next_sibling_hdr(hdr_field_t *hf)
 	return NULL;
 }
 
-hdr_field_t* get_hdr_by_name(sip_msg_t *msg, char *name, int name_len)
+hdr_field_t* get_hdr_by_name(const sip_msg_t* const msg, const char* const name, const int name_len)
 {
 	hdr_field_t *hdr;
 
@@ -855,8 +961,8 @@ hdr_field_t* get_hdr_by_name(sip_msg_t *msg, char *name, int name_len)
 	return NULL;
 }
 
-
-hdr_field_t* next_sibling_hdr_by_name(hdr_field_t *hf)
+/** not used yet */
+hdr_field_t* next_sibling_hdr_by_name(const hdr_field_t* const hf)
 {
 	hdr_field_t *hdr;
 
@@ -872,7 +978,7 @@ hdr_field_t* next_sibling_hdr_by_name(hdr_field_t *hf)
  * set msg context id
  * - return: -1 on error; 0 - on set
  */
-int msg_ctx_id_set(sip_msg_t *msg, msg_ctx_id_t *mid)
+int msg_ctx_id_set(const sip_msg_t* const msg, msg_ctx_id_t* const mid)
 {
 	if(msg==NULL || mid==NULL)
 		return -1;
@@ -885,7 +991,7 @@ int msg_ctx_id_set(sip_msg_t *msg, msg_ctx_id_t *mid)
  * check msg context id
  * - return: -1 on error; 0 - on no match; 1 - on match
  */
-int msg_ctx_id_match(sip_msg_t *msg, msg_ctx_id_t *mid)
+int msg_ctx_id_match(const sip_msg_t* const msg, const msg_ctx_id_t* const mid)
 {
 	if(msg==NULL || mid==NULL)
 		return -1;
@@ -897,7 +1003,7 @@ int msg_ctx_id_match(sip_msg_t *msg, msg_ctx_id_t *mid)
 /**
  * set msg time value
  */
-int msg_set_time(sip_msg_t *msg)
+int msg_set_time(sip_msg_t* const msg)
 {
 	if(unlikely(msg==NULL))
 		return -2;
@@ -905,3 +1011,133 @@ int msg_set_time(sip_msg_t *msg)
 		return 0;
 	return gettimeofday(&msg->tval, NULL);
 }
+
+/**
+ * get source ip, port and protocol in SIP URI format
+ * - tmode - 0: short format (transport=udp is not added, being default)
+ */
+int get_src_uri(sip_msg_t *m, int tmode, str *uri)
+{
+	static char buf[MAX_URI_SIZE];
+	char* p;
+	str ip, port;
+	int len;
+	str proto;
+
+	if (!uri || !m) {
+		ERR("invalid parameter value\n");
+		return -1;
+	}
+
+	if(tmode==0) {
+		switch(m->rcv.proto) {
+			case PROTO_NONE:
+			case PROTO_UDP:
+				proto.s = 0; /* Do not add transport parameter, UDP is default */
+				proto.len = 0;
+			break;
+			default:
+				if(get_valid_proto_string(m->rcv.proto, 1, 0, &proto)<0) {
+					ERR("unknown transport protocol\n");
+					return -1;
+				}
+		}
+	} else {
+		if(get_valid_proto_string(m->rcv.proto, 1, 0, &proto)<0) {
+			ERR("unknown transport protocol\n");
+			return -1;
+		}
+	}
+
+	ip.s = ip_addr2a(&m->rcv.src_ip);
+	ip.len = strlen(ip.s);
+
+	port.s = int2str(m->rcv.src_port, &port.len);
+
+	len = 4 + ip.len + 2*(m->rcv.src_ip.af==AF_INET6)+ 1 + port.len;
+	if (proto.s) {
+		len += TRANSPORT_PARAM_LEN;
+		len += proto.len;
+	}
+
+	if (len > MAX_URI_SIZE) {
+		ERR("buffer too small\n");
+		return -1;
+	}
+
+	p = buf;
+	memcpy(p, "sip:", 4);
+	p += 4;
+
+	if (m->rcv.src_ip.af==AF_INET6)
+		*p++ = '[';
+	memcpy(p, ip.s, ip.len);
+	p += ip.len;
+	if (m->rcv.src_ip.af==AF_INET6)
+		*p++ = ']';
+
+	*p++ = ':';
+
+	memcpy(p, port.s, port.len);
+	p += port.len;
+
+	if (proto.s) {
+		memcpy(p, TRANSPORT_PARAM, TRANSPORT_PARAM_LEN);
+		p += TRANSPORT_PARAM_LEN;
+
+		memcpy(p, proto.s, proto.len);
+		p += proto.len;
+	}
+
+	uri->s = buf;
+	uri->len = len;
+
+	return 0;
+}
+
+/*! \brief returns a pointer to the begining of the msg's body
+ */
+char* get_body(sip_msg_t* const msg)
+{
+	int offset;
+	unsigned int len;
+
+	if ( parse_headers(msg, HDR_EOH_F, 0)==-1 ) {
+		LM_ERR("failed to parse to end of headers\n");
+		return 0;
+	}
+
+	if (msg->unparsed) {
+		len=(unsigned int)(msg->unparsed-msg->buf);
+	} else {
+		LM_ERR("unparsed hook for end of headers is not set\n");
+		return 0;
+	}
+
+	if ((len+2<=msg->len) && (strncmp(CRLF,msg->unparsed,CRLF_LEN)==0) ) {
+		offset = CRLF_LEN;
+	} else if ( (len+1<=msg->len) &&
+				(*(msg->unparsed)=='\n' || *(msg->unparsed)=='\r' ) ) {
+		offset = 1;
+	} else {
+		LM_ERR("failed to locate end of headers (%p %p - %d %d [%s])\n",
+				msg->buf, msg->unparsed, msg->len, len, msg->unparsed);
+		return 0;
+	}
+
+	return msg->unparsed + offset;
+}
+
+/*! \brief make sure all HFs needed for transaction identification have been
+ * parsed; return 0 if those HFs can't be found
+*/
+int check_transaction_quadruple(sip_msg_t* const msg)
+{
+	if ( parse_headers(msg, HDR_FROM_F|HDR_TO_F|HDR_CALLID_F|HDR_CSEQ_F,0)!=-1
+		&& msg->from && msg->to && msg->callid && msg->cseq ) {
+		return 1;
+	} else {
+		ser_error=E_BAD_TUPEL;
+		return 0;
+	}
+}
