@@ -1,24 +1,16 @@
 /*
- * $Id$
- *
  * SIP routing engine
  *
- *
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -27,37 +19,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * History:
- * --------
- *  2003-01-28  scratchpad removed, src_port introduced (jiri)
- *  2003-02-28  scratchpad compatibility abandoned (jiri)
- *  2003-03-10  updated to the new module exports format (andrei)
- *  2003-03-19  replaced all mallocs/frees w/ pkg_malloc/pkg_free (andrei)
- *  2003-04-01  added dst_port, proto, af; renamed comp_port to comp_no,
- *               inlined all the comp_* functions (andrei)
- *  2003-04-05  s/reply_route/failure_route, onreply_route introduced (jiri)
- *  2003-05-23  comp_ip fixed, now it will resolve its operand and compare
- *              the ip with all the addresses (andrei)
- *  2003-10-10  added more operators support to comp_* (<,>,<=,>=,!=) (andrei)
- *  2004-10-19  added from_uri & to_uri (andrei)
- *  2005-12-12  added retcode support (anrei)
- *  2005-12-19  select framework (mma)
- *  2006-01-30  removed rec. protection from eval_expr (andrei)
- *  2006-02-06  added named route tables (andrei)
- *  2008-04-14  (expr1 != expr2) is evaluated true if at least one of
- *		the expressions does not exist (Miklos)
- *  2008-04-23  errors are treated as false during expression evaluation
- *  		unless the operator is DIFF_OP (Miklos)
- *  2008-12-03  fixups for rvalues in assignments (andrei)
- *  2009-05-04  switched IF_T to rval_expr (andrei)
- *  2010-06-01  special hack/support for fparam fixups so that they can handle
- *               variable RVEs (andrei)
- *  2010-06-18  ip comparison (comp_ip()) normalizes strings to
- *              ip/netmask  (andrei)
  */
 
 
-/** expression evaluation, route fixups and routing lists.
+/** Kamailio core :: expression evaluation, route fixups and routing lists.
  * @file route.c
  * @ingroup core
  * Module: @ref core
@@ -405,7 +370,7 @@ static int exp_optimize_left(struct expr* exp)
 		}
 	}
 	if (ret>0)
-		DBG("left EXP optimized: op%d(_O%d_, ST%d) => op%d(_O%d_, ST%d)\n",
+		LM_DBG("op%d(_O%d_, ST%d) => op%d(_O%d_, ST%d)\n",
 			old_op, old_ltype, old_rtype, exp->op, exp->l_type, exp->r_type);
 	return ret;
 }
@@ -503,7 +468,7 @@ static int exp_optimize_right(struct expr* exp)
 		}
 	}
 	if (ret>0)
-		DBG("right EXP optimized: op%d(O%d, _ST%d_) => op%d(O%d, _ST%d_)\n",
+		LM_DBG("op%d(O%d, _ST%d_) => op%d(O%d, _ST%d_)\n",
 			old_op, old_ltype, old_rtype, exp->op, exp->l_type, exp->r_type);
 	return ret;
 }
@@ -904,7 +869,7 @@ int fix_actions(struct action* a)
 				cmd = t->val[0].u.data;
 				rve_param_no = 0;
 				if (cmd) {
-					DBG("fixing %s()\n", cmd->name);
+					LM_DBG("fixing %s()\n", cmd->name);
 					if (t->val[1].u.number==0) {
 						ret = call_fixup(cmd->fixup, 0, 0);
 						if (ret < 0)
@@ -1028,7 +993,7 @@ int fix_actions(struct action* a)
 				si=find_si(&ip, ((struct socket_id*)t->val[0].u.data)->port,
 								((struct socket_id*)t->val[0].u.data)->proto);
 				if (si==0){
-					LM_ERR("bad force_send_socket argument: %s:%d (ser doesn't listen on it)\n",
+					LM_ERR("bad force_send_socket argument: %s:%d (" NAME " doesn't listen on it)\n",
 						((struct socket_id*)t->val[0].u.data)->addr_lst->name,
 							((struct socket_id*)t->val[0].u.data)->port);
 					ret = E_BAD_ADDRESS;
@@ -1735,7 +1700,7 @@ inline static int comp_ip(int op, struct ip_addr* ip, int rtype,
 			/* 2: resolve (name) & compare w/ all the ips */
 			he=resolvehost(right->s);
 			if (he==0){
-				DBG("comp_ip: could not resolve %s\n", r->str.s);
+				LM_DBG("could not resolve %s\n", r->str.s);
 			}else if (he->h_addrtype==ip->af){
 				for(h=he->h_addr_list;(ret!=1)&& (*h); h++){
 					ret=(memcmp(ip->u.addr, *h, ip->len)==0);
@@ -1770,7 +1735,7 @@ inline static int comp_ip(int op, struct ip_addr* ip, int rtype,
 			/* 2: resolve (name) & compare w/ all the ips */
 			he=resolvehost(right->s);
 			if (he==0){
-				DBG("comp_ip: could not resolve %s\n", r->str.s);
+				LM_DBG("could not resolve %s\n", r->str.s);
 			}else if (he->h_addrtype==ip->af){
 				for(h=he->h_addr_list;(ret!=1)&& (*h); h++){
 					ret=(memcmp(ip->u.addr, *h, ip->len)==0);
@@ -2161,10 +2126,10 @@ static void print_rl(struct route_list* rt, char* name)
 	for(j=0; j<rt->entries; j++){
 		if (rt->rlist[j]==0){
 			if ((j==0) && (rt==&main_rt))
-				DBG("WARNING: the main routing table is empty\n");
+				LM_DBG("WARNING: the main routing table is empty\n");
 			continue;
 		}
-		DBG("%s routing table %d:\n", name, j);
+		LM_DBG("%s routing table %d:\n", name, j);
 		print_actions(rt->rlist[j]);
 		DBG("\n");
 	}
